# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --
from nagare import presentation, component, database, security, i18n
from ...user import usermanager
from ..database.forms import RegistrationTask
from . import oauth_providers
from kansha.services.authentication_repository import Authentication


class OAuthConnection(object):
    def __init__(self, provider):
        self.provider = provider
        self.content = component.Component()  # workaround nagare weird behavior of call if on_answer registered on this component

    @property
    def source(self):
        return self.provider.name

    def trigger_provider(self, comp):
        r = self.content.call(self.provider)  # here is the trick
        comp.answer(r)


@presentation.render_for(OAuthConnection)
def render_connect(self, h, comp, *args):
    if self.content() is not None:
        return self.content
    else:
        return comp.render(h, 'button')


@presentation.render_for(OAuthConnection, 'button')
def render_button(self, h, comp, *args):
    h << h.a(
        i18n._('Sign in with %s') % self.source.capitalize(),
        class_="oauth " + self.source
    ).action(self.trigger_provider, comp)
    return h.root


SCOPES = {
    'google': ('profile', 'email'),
    'facebook': ('public_profile', 'email'),
    'twitter': ('profile', 'email'),
    'github': ('user:email',)
}


class Login(Authentication):
    alt_title = None
    CONFIG_SPEC = {
        'activated': 'boolean(default=False)',
    }
    oauth_config_spec = {
        'activated': 'boolean',
        'key': 'string(default="")',
        'secret': 'string(default="")'
    }

    def __init__(self, app_title, app_banner, theme, services_service):
        self.oauth_modules = {}
        self._error_message = u''
        self.services_service = services_service
        self.app_title = app_title
        self.app_banner = app_banner
        self.theme = theme
        self.content = component.Component()  # workaround nagare weird behavior of call if on_answer registered on this component

        for source, cfg in self.config.iteritems():
            try:
                if cfg['activated'] == 'on':
                    self.oauth_modules[source] = component.Component(
                        OAuthConnection(
                            oauth_providers.providers[source](
                                cfg['key'],
                                cfg['secret'],
                                SCOPES.get(source, ('profile', 'email'))
                            )
                        )
                    )
            except TypeError:
                # source is not a provider entry
                continue

    @property
    def error_message(self):
        return self._error_message or getattr(self.content(), 'error_message', u'')

    @error_message.setter
    def error_message(self, value):
        self._error_message = value
        if self.content():
            setattr(self.content(), 'error_message', u'')

    def connect(self, comp, oauth_user, source):
        if oauth_user is None:
            self._error_message = i18n._(u'Authentication failed')
            return

        profile = oauth_user.get_profile()[0]

        profile_id = profile['id']
        # if user exists update data
        name = profile['name'] or (i18n._(u'Please provide a full name in %s') % source)
        if not usermanager.UserManager.get_by_username(profile_id):
            usermanager.UserManager().create_user(profile['id'], None, name, profile['email'], source=source,
                                                  picture=profile.get('picture'))

        # update takes care of not overwriting the existing email with an empty one
        usermanager.UserManager.get_by_username(profile_id).update(name, profile['email'],
                                                                   picture=profile.get('picture'))
        # thus if data_user.email is empty, that means it has always been so.
        if not usermanager.UserManager.get_by_username(profile_id).email:
            self.content.call(
                self.services_service(
                    RegistrationTask,
                    self.app_title,
                    self.app_banner,
                    self.theme,
                    username=profile_id
                )
            )
            return
        database.session.flush()
        u = usermanager.UserManager.get_app_user(profile['id'], data=usermanager.UserManager.get_by_username(profile_id))
        security.set_user(u)

        # After a successful OAuth authentication, we need to manually switch to the user's locale
        # so the first rendering phase uses the correct language
        i18n.set_locale(u.get_locale())

        comp.answer(u)

@presentation.render_for(Login)
def render_login(self, h, comp, *args):
    if self.content() is not None:
        return self.content
    else:
        return comp.render(h, 'buttons')


@presentation.render_for(Login, 'buttons')
def render_buttons(self, h, comp, *args):
    with h.div(class_='login oauthLogin'):
        for (source, oauth) in self.oauth_modules.items():
            h << oauth.on_answer(lambda u, source=source: self.connect(comp, u, source))
    return h.root



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import absolute_import

from elixir import EntityBase
from elixir import EntityMeta

from kansha import pickle


class Entity(EntityBase, pickle.UnpicklableMixin):
    __metaclass__ = EntityMeta

    @classmethod
    def exists(cls, **kw):
        return cls.query.filter_by(**kw).count() > 0



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from elixir import Boolean
from elixir import Field
from elixir import Integer
from elixir import ManyToOne
from elixir import OneToMany
from elixir import Unicode
from elixir import using_options
from sqlalchemy import func
from sqlalchemy.ext.orderinglist import ordering_list

from nagare import database

from kansha.models import Entity


class DataChecklist(Entity):
    using_options(tablename='checklists')

    title = Field(Unicode(255))
    items = OneToMany('DataChecklistItem', order_by='index', inverse='checklist',
                      collection_class=ordering_list('index'))
    card = ManyToOne('DataCard')
    author = ManyToOne('DataUser')
    index = Field(Integer)

    @classmethod
    def get_by_card(cls, card):
        q = cls.query
        q = q.filter_by(card=card)
        q = q.order_by(cls.index)
        return q.all()

    def update(self, other):
        self.title = other.title
        self.index = other.index
        for item in other.items:
            self.items.append(DataChecklistItem(title=item.title,
                                                index=item.index,
                                                done=False))
        database.session.flush()

    def __unicode__(self):
        titles = [item.title for item in self.items if item.title]
        if self.title:
            titles.insert(0, self.title)
        return u'\n'.join(titles)

    def to_indexable(self):
        return unicode(self)

    def add_item_from_str(self, text):
        item = DataChecklistItem.new_from_str(text)
        return self.add_item(item)

    def add_item(self, item):
        self.items.append(item)
        return item

    def insert_item(self, index, item):
        self.items.insert(index, item)

    def remove_item(self, item):
        self.items.remove(item)

    def delete_item(self, item):
        self.remove_item(item)
        item.delete()

    def purge(self):
        for item in self.items:
            item.delete()

    @staticmethod
    def total_items(card):
        return DataChecklistItem.total_items(card)

    @staticmethod
    def total_items_done(card):
        return DataChecklistItem.total_items_done(card)


class DataChecklistItem(Entity):
    using_options(tablename='checklist_items')

    index = Field(Integer)
    title = Field(Unicode(255))
    done = Field(Boolean)
    checklist = ManyToOne('DataChecklist')

    @classmethod
    def new_from_str(cls, text):
        item = cls(title=text.strip())
        database.session.flush()
        return item

    @classmethod
    def total_items(cls, card):
        # query.count() is sloooow, so we use an alternate method
        q = cls.query.join(DataChecklist).filter(DataChecklist.card == card)
        return q.with_entities(func.count()).scalar()

    @classmethod
    def total_items_done(cls, card):
        # query.count() is sloooow, so we use an alternate method
        q = cls.query.join(DataChecklist).filter(DataChecklist.card == card).filter(
            DataChecklistItem.done == True)
        return q.with_entities(func.count()).scalar()



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --
from nagare import log

try:
    import ldap
except ImportError:
    ldap = None
import types


def toUTF8(v):
    if isinstance(v, unicode):
        return v.encode('utf-8')
    elif isinstance(v, (types.TupleType, types.ListType)):
        return [toUTF8(e) for e in v]
    elif isinstance(v, types.DictType):
        return dict([(toUTF8(k), toUTF8(v_)) for k, v_ in v.items()])
    else:
        return v


class LDAPAuth(object):
    def __init__(self, ldap_cfg):
        ldap_cfg = toUTF8(ldap_cfg)
        self.server = "ldap://%s:%s" % (ldap_cfg['host'], ldap_cfg['port'])
        self.users_base_dn = ldap_cfg['users_base_dn']

    def connect(self):
        """Connect to LDAP server

        Return:
            - a server connection
        """
        assert ldap, 'python_ldap not installed'
        return ldap.initialize(self.server)

    def get_user_dn(self, uid):
        raise NotImplementedError()

    def check_password(self, uid, password):
        """Check if the specified couple user/password is correct

        In:
           - ``uid`` -- the user id
           - ``password`` -- the user password
        Return:
            - True if password is checked
        """
        c = self.connect()
        dn = self.get_user_dn(uid)
        # Try to authenticate
        try:
            c.simple_bind_s(dn, password.encode('UTF-8'))
            return True
        except ldap.INVALID_CREDENTIALS:
            log.info("Bad credentials for DN %r" % dn)
        except ldap.SERVER_DOWN:
            log.critical("LDAP server down")
        finally:
            c.unbind()

    def get_profile(self, uid, password):
        raise NotImplementedError()


class NngLDAPAuth(LDAPAuth):
    def get_user_dn(self, uid):
        """Construct a user DN given an user id

        In:
            - ``uid`` -- the user id
        Return:
            - a string, the user DN
        """
        return 'uid=%s,%s' % (ldap.dn.escape_dn_chars(toUTF8(uid)), self.users_base_dn)

    def get_profile(self, uid, password):
        c = self.connect()
        ldap_result = c.search_s(self.get_user_dn(uid), ldap.SCOPE_BASE)[0][1]
        profile = {}
        profile['uid'] = ldap_result['uid'][0]
        profile['name'] = ldap_result['displayName'][0].decode('utf-8')
        profile['email'] = ldap_result['mail'][0]
        profile['picture'] = ldap_result['jpegPhoto'][0] if ('jpegPhoto' in ldap_result and
                                                             ldap_result['jpegPhoto']) else None
        c.unbind()
        return profile


class ADLDAPAuth(LDAPAuth):
    def connect(self):
        conn = super(ADLDAPAuth, self).connect()
        conn.set_option(ldap.OPT_REFERRALS, 0)
        conn.protocol_version = 3
        return conn

    def check_password(self, uid, password):
        """Check if the specified couple user/password is correct

        In:
           - ``uid`` -- the user id
           - ``password`` -- the user password
        Return:
            - True if password is checked
        """
        c = self.connect()
        # Try to authenticate
        try:
            c.simple_bind_s(uid, password)
            return True
        except ldap.INVALID_CREDENTIALS:
            log.info("Bad credentials for uid %r" % uid)
        except ldap.SERVER_DOWN:
            log.critical("LDAP server down")
        finally:
            c.unbind()

    def get_profile(self, uid, password):
        c = self.connect()
        c.simple_bind_s(uid, password)
        ldap_result = c.search_s(self.users_base_dn, ldap.SCOPE_SUBTREE,
                                 '(userPrincipalName=%s)' % ldap.dn.escape_dn_chars(toUTF8(uid)))[0][1]
        profile = {}
        profile['uid'] = ldap_result['sAMAccountName'][0]
        profile['name'] = ldap_result['displayName'][0].decode('utf-8')
        profile['email'] = ldap_result.get('mail', [''])[0]
        profile['picture'] = ldap_result['thumbnailPhoto'][0] if ('thumbnailPhoto' in ldap_result and
                                                                  ldap_result['thumbnailPhoto']) else None
        c.unbind()
        return profile



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from kansha.cardextension.tests import CardExtensionTestCase

from .comp import Votes


class VoteTest(CardExtensionTestCase):

    extension_name = 'votes'
    extension_class = Votes

    def test_toggle(self):
        self.extension.toggle()
        self.assertEqual(self.extension.count_votes(), 1)
        self.assertTrue(self.extension.has_voted())
        self.extension.toggle()
        self.assertEqual(self.extension.count_votes(), 0)
        self.assertFalse(self.extension.has_voted())



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

# TODO: make it a board extension


class BoardCardFilter(object):

    def __init__(self, card_schema, board_id, exclude_archived, search_engine_service):
        self.card_schema = card_schema
        self.board_id = board_id
        self._exclude_archived = exclude_archived
        self.search_engine = search_engine_service
        self.last_search = ''
        self.card_matches = set()
        # changed by external API call?
        self._changed = False

    def exclude_archived(self, flag):
        self._exclude_archived = flag

    def search(self, query):
        self.last_search = query
        if query:
            condition = self.card_schema.match(query) & (self.card_schema.board_id == self.board_id)
            # do not query archived cards if archive column is hidden
            if self._exclude_archived:
                condition &= (self.card_schema.archived == False)
            self.card_matches = set(doc._id for (_, doc) in self.search_engine.search(condition, 1000))
            # make the difference between empty search and no results
            if not self.card_matches:
                self.card_matches.add(None)
        else:
            self.card_matches = set()

    def reset(self):
        self.last_search = ''
        self.card_matches = set()
        self._changed = True

    def __call__(self, card):
        """Return "highlight" if card matches, "hidden" if it doesn't or "" if
        no filter is active."""
        status = ''
        if self.card_matches:
            status = 'highlight' if card.id in self.card_matches else 'hidden'
        return status

    def reload_search(self):
        if self.last_search:
            self.search(self.last_search)



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Add versioning

Revision ID: d0e88000d3f
Revises: b740362087
Create Date: 2013-10-30 10:45:11.526568

"""

# revision identifiers, used by Alembic.
revision = 'd0e88000d3f'
down_revision = 'b740362087'

from alembic import context, op
from nagare import database
import sqlalchemy as sa

from kansha.board import models


def upgrade():
    op.add_column('board', sa.Column('version', sa.Integer, server_default='0'))


def downgrade():
    op.drop_column('board', 'version')



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

from nagare import presentation, ajax
from nagare.i18n import format_datetime, _

from .comp import ActionLog


@presentation.render_for(ActionLog)
def render_ActionLog(self, h, comp, *args):
    return comp.render(h.AsyncRenderer(), 'history')


# FIXME: demeter, foreign data direct access
@presentation.render_for(ActionLog, 'history')
def render_ActionLog_history(self, h, *_args):
    h << h.h2(_('Action log'))
    log = self.get_history()
    users = set(event.user for event in log)
    cards = set(event.card for event in log)
    with h.div(id='action-log'):
        with h.form:
            with h.select(onchange=ajax.Update(action=self.user_id)):
                h << h.option(_('all users'), value='')
                for member in sorted(users, key=lambda u: u.fullname):
                    h << h.option(member.fullname, value=member.username).selected(
                        self.user_id())
            with h.select(onchange=ajax.Update(action=lambda x: self.card_id(int(x)))):
                h << h.option(_('all cards'), value=0)
                for card in sorted(cards, key=lambda c: c.title):
                    h << h.option(card.title, value=card.id).selected(
                        self.card_id())
        with h.div(class_='history'):
            with h.table(class_='table table-striped table-hover'):
                with h.body:
                    for event in self.get_history():
                        with h.tr:
                            h << h.th(format_datetime(event.when, 'short'))
                            h << h.td(event.to_string())
    return h.root



"""email index

Revision ID: 55f89221fc55
Revises: 25739bc150b9
Create Date: 2017-01-24 16:44:02.500510

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import and_

# revision identifiers, used by Alembic.
revision = '55f89221fc55'
down_revision = '25739bc150b9'


users = sa.Table(
    'user',
    sa.MetaData(),
    sa.Column('username', sa.VARCHAR(255), primary_key=True),
    sa.Column('source', sa.VARCHAR(255), primary_key=True),
    sa.Column('email', sa.Unicode),
    sa.Column('email_to_confirm', sa.Unicode),
    sa.Column('registration_date', sa.DateTime)
)

membership = sa.Table(
    'membership',
    sa.MetaData(),
    sa.Column('user_username', sa.Unicode, nullable=False),
    sa.Column('user_source', sa.Unicode, nullable=False),
)

vote = sa.Table(
    'vote',
    sa.MetaData(),
    sa.Column('user_username', sa.Unicode, nullable=False),
    sa.Column('user_source', sa.Unicode, nullable=False)
)

history = sa.Table(
    'history',
    sa.MetaData(),
    sa.Column('user_username', sa.Unicode, nullable=False),
    sa.Column('user_source', sa.Unicode, nullable=False)
)

checklist = sa.Table(
    'checklists',
    sa.MetaData(),
    sa.Column('author_username', sa.Unicode, nullable=False),
    sa.Column('author_source', sa.Unicode, nullable=False)
)

comment = sa.Table(
    'comment',
    sa.MetaData(),
    sa.Column('author_username', sa.Unicode, nullable=False),
    sa.Column('author_source', sa.Unicode, nullable=False)
)

asset = sa.Table(
    'asset',
    sa.MetaData(),
    sa.Column('author_username', sa.Unicode, nullable=False),
    sa.Column('author_source', sa.Unicode, nullable=False)
)

card = sa.Table(
    'card',
    sa.MetaData(),
    sa.Column('author_username', sa.Unicode, nullable=False),
    sa.Column('author_source', sa.Unicode, nullable=False)
)

board = sa.Table(
    'board',
    sa.MetaData(),
    sa.Column('last_users_username', sa.Unicode, nullable=False),
    sa.Column('last_users_source', sa.Unicode, nullable=False)
)


def merge_users(bind, old_user, new_user):
    print "merging", old_user, 'into', new_user
    bind.execute(
        membership.update().where(
            and_(
                membership.c.user_username == old_user.username,
                membership.c.user_source == old_user.source
            )
        ).values(
            user_username=new_user.username,
            user_source=new_user.source
        )
    )
    bind.execute(
        vote.update().where(
            and_(
                vote.c.user_username == old_user.username,
                vote.c.user_source == old_user.source
            )
        ).values(
            user_username=new_user.username,
            user_source=new_user.source
        )
    )
    bind.execute(
        history.update().where(
            and_(
                history.c.user_username == old_user.username,
                history.c.user_source == old_user.source
            )
        ).values(
            user_username=new_user.username,
            user_source=new_user.source
        )
    )
    bind.execute(
        comment.update().where(
            and_(
                comment.c.author_username == old_user.username,
                comment.c.author_source == old_user.source
            )
        ).values(
            author_username=new_user.username,
            author_source=new_user.source
        )
    )
    bind.execute(
        checklist.update().where(
            and_(
                checklist.c.author_username == old_user.username,
                checklist.c.author_source == old_user.source
            )
        ).values(
            author_username=new_user.username,
            author_source=new_user.source
        )
    )
    bind.execute(
        asset.update().where(
            and_(
                asset.c.author_username == old_user.username,
                asset.c.author_source == old_user.source
            )
        ).values(
            author_username=new_user.username,
            author_source=new_user.source
        )
    )
    bind.execute(
        card.update().where(
            and_(
                card.c.author_username == old_user.username,
                card.c.author_source == old_user.source
            )
        ).values(
            author_username=new_user.username,
            author_source=new_user.source
        )
    )
    bind.execute(
        board.update().where(
            and_(
                board.c.last_users_username == old_user.username,
                board.c.last_users_source == old_user.source
            )
        ).values(
            last_users_username=new_user.username,
            last_users_source=new_user.source
        )
    )


def upgrade():
    # first, find and merge duplicates
    # then, set unique index
    bind = op.get_bind()
    select = sa.text('select email from "user" where email is not null group by email having count(*) > 1')
    for email in bind.execute(select):
        same_users = bind.execute(users.select().where(users.c.email == email[0]).order_by('registration_date')).fetchall()
        kept_user = same_users.pop()
        for obsolete_user in same_users:
            merge_users(bind, obsolete_user, kept_user)
            bind.execute(users.delete().where(
                and_(
                    users.c.username == obsolete_user.username,
                    users.c.source == obsolete_user.source)))
    # phantom users, lost forever...
    bind.execute(users.update().where(users.c.email == None).values(email_to_confirm=None))
    with op.batch_alter_table('user', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_email'), ['email'], unique=True)
        batch_op.create_index(batch_op.f('ix_email_to_confirm'), ['email_to_confirm'], unique=True)


def downgrade():
    pass



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import time
import hashlib
import textwrap
from datetime import datetime, timedelta

import webob
from nagare.i18n import _
from nagare import (presentation, editor, component, security, log, database)

from . import captcha
from kansha import validator
from kansha.user import usermanager
from kansha.user.models import DataToken
from kansha.services.authentication_repository import Authentication


UserConfirmationTimeout = timedelta(hours=12)


class Header(object):

    def __init__(self, app_title, app_banner, theme):
        self.app_title = app_title
        self.app_banner = app_banner
        self.theme = theme


@presentation.render_for(Header)
def render_Header(self, h, comp, *args):
    """Head renderer"""

    h.head << h.head.title(self.app_banner)
    h.head << h.head.meta(
        name='viewport', content='width=device-width, initial-scale=1.0')

    h.head.css_url('css/knacss.css')
    h.head.css_url('css/themes/fonts.css?v=2c')
    h.head.css_url('css/themes/kansha.css')
    h.head.css_url('css/themes/login.css?v=2c')
    h.head.css_url('css/themes/%s/kansha.css?v=2c' % self.theme)
    h.head.css_url('css/themes/%s/login.css?v=2c' % self.theme)

    with h.div(class_='header'):
        with h.a(href=h.request.application_url):
            h << h.div(class_='logo')
        h << h.h1(self.app_banner)
    return h.root


@presentation.render_for(Header, 'hide')
def render_Header_hide(self, h, comp, *args):
    h.head.css('hide_logins', u'''.login {display:none;}''')
    return h.root


def redirect_to(url):
    raise webob.exc.HTTPSeeOther(location=url)


class Login(Authentication):

    CONFIG_SPEC = {
        'activated': 'boolean(default=True)',
        'moderator': 'string(default="")',
        'identicons': 'boolean(default=False)',
        'default_username': 'string(default="")',
        'default_password': 'string(default="")'
    }

    def __init__(self, app_title, app_banner, theme, assets_manager_service, services_service):
        self.assets_manager = assets_manager_service
        self._error_message = ''
        self.registration_task = services_service(RegistrationTask, app_title, app_banner,
                                                  theme, moderator=self.config['moderator'],
                                                  identicons=self.config['identicons'])
        self.default_username = self.config['default_username']
        self.default_password = self.config['default_password']
        self.pwd_reset = services_service(PasswordResetTask, app_title, app_banner, theme)
        self.content = component.Component()

    @property
    def alt_title(self):
        'Return a unicode to overwrite default login page title, or None.'
        return None or getattr(self.content(), 'alt_title', None)

    @property
    def error_message(self):
        return self._error_message or getattr(self.content(), 'error_message', u'')

    @error_message.setter
    def error_message(self, value):
        self._error_message = value
        if self.content():
            setattr(self.content(), 'error_message', u'')

    def log_in(self, comp):
        u = security.get_user()
        # If user is not local user remove user from security
        if u and not u.is_local:
            security.set_user(None)
            u = None
        if u is not None:
            database.session.flush()
            comp.answer(u)
        else:
            self._error_message = _('Login failed')


@presentation.render_for(Login)
def render_Login(self, h, comp, *args):
    if self.content() is not None:
        return self.content
    else:
        return comp.render(h, 'form')


@presentation.render_for(Login, 'form')
def render_Login_form(self, h, comp, *args):
    with h.div(class_='login databaseLogin'):
        with h.form:
            # if self.error_message:
            #     h << h.br << h.div(self.error_message, class_="error")
            h << h.input(type='text', name='__ac_name', id='username',
                         value=self.default_username, placeholder=_('Enter username'))
            h << h.input(type='password', name='__ac_password', id="password",
                         value=self.default_password, placeholder=_('Enter password'))
            h << h.a(_('Forgot password?')).action(self.content.call, self.pwd_reset)
            with h.div(class_='actions'):
                h << h.input(type='submit', value=_(u'Sign in'), class_='btn btn-primary').action(self.log_in, comp)
                with h.div:
                    h << _('No account yet? ')
                    h << h.a(_('Sign up')).action(self.content.call, self.registration_task)
    return h.root


class RegistrationForm(editor.Editor):

    """Registration form for creating a new (unconfirmed) user"""

    def __init__(self, app_title, app_banner, theme):
        self.username = editor.Property('').validate(self.validate_username)
        self.email = editor.Property('').validate(self.validate_email)
        self.fullname = editor.Property('').validate(validator.validate_non_empty_string)
        self.password = editor.Property('').validate(validator.validate_password)
        self.password_repeat = editor.Property('').validate(validator.validate_password)
        self.init_captcha_image()
        self.captcha_text = editor.Property('').validate(self.validate_captcha)
        self.header = component.Component(Header(app_title, app_banner, theme))
        self.user_manager = usermanager.UserManager()
        self.error_message = u''
        self.alt_title = _(u'Sign up')

    def init_captcha_image(self):
        self.captcha_image = component.Component(captcha.Captcha())
        self.captcha_date = datetime.now()

    def validate_username(self, value):
        value = validator.validate_identifier(value)
        # check that this user name does not exist
        u = usermanager.UserManager.get_by_username(value)
        if u:
            raise ValueError(_("Username %s is not available. Please choose another one.")
                             % value)
        return value

    def validate_email(self, value):
        validator.validate_email(value)
        u = usermanager.UserManager.get_by_email(value)
        if u:
            raise ValueError(_(u"This email address (%s) is already registered.")
                             % value)
        return value

    def validate_captcha(self, value):
        # validate the text
        if not value or (value.lower() != self.captcha_image().text.lower()):
            self.init_captcha_image()
            raise ValueError(_("Invalid captcha"))

        # check the expiration date
        captcha_expiration = timedelta(minutes=10)
        if (datetime.now() - self.captcha_date) > captcha_expiration:
            self.init_captcha_image()
            raise ValueError(_("Entered too late"))
        return value

    def validate_passwords_match(self):
        if self.password.value != self.password_repeat.value:
            self.password_repeat.error = _("The two passwords don't match")

    def commit(self):

        properties = ('username', 'fullname', 'email', 'password',
                      'password_repeat', 'captcha_text')
        if not self.is_validated(properties):
            self.captcha_image.becomes(captcha.Captcha())
            self.error_message = _(u'Unable to process. Check your input below.')
            return None

        # register the user in the database
        u = self.user_manager.create_user(username=self.username.value,
                                          password=self.password.value,
                                          fullname=self.fullname.value,
                                          email=self.email.value)
        return u

    def on_ok(self, comp, application_url):
        u = self.commit()
        if u:
            comp.answer((u.username, application_url))


@presentation.render_for(RegistrationForm)
def render_RegistrationForm(self, h, comp, *args):
    h << self.header.render(h, 'hide')
    with h.div(class_='regForm'):
        # autocomplete="off": do not store the password
        with h.form(autocomplete="off").post_action(self.validate_passwords_match):
            with h.div(class_='fields'):
                fields = (
                    (_('Username'), 'username', 'text', self.username),
                    (_('Password'),
                     'password', 'password', self.password),
                    (_('Password (repeat)'), 'password-repeat',
                     'password', self.password_repeat),
                    (_('Email address'), 'email', 'text', self.email),
                    (_('Fullname'), 'fullname', 'text', self.fullname)
                )
                for label, css_class, input_type, property in fields:
                    with h.div(class_='%s-field field' % css_class):
                        id_ = h.generate_id("field")
                        with h.label(for_=id_):
                            h << label
                        h << h.input(id=id_,
                                     type=input_type,
                                     value=property()).action(property).error(property.error)

                with h.div(class_='captcha-field field'):
                    id_ = h.generate_id("field")
                    with h.label(for_=id_):
                        h << _("Enter the captcha text")
                    h << h.input(id=id_,
                                 type='text').action(self.captcha_text).error(self.captcha_text.error)
                h << comp.render(h.AsyncRenderer(), 'captcha')
            with h.div(class_='actions'):
                h << h.input(type='submit',
                             value=_("Create new account"),
                             class_="btn btn-primary").action(self.on_ok, comp, h.request.application_url)

            h << _("Already have an account? ") << h.a(
                _("Log in")).action(comp.answer, (None, None))

    return h.root


@presentation.render_for(RegistrationForm, 'captcha')
def render_RegistrationForm_captcha(self, h, comp, *args):
    with h.div(id='captchaImage'):
        h << self.captcha_image
        h << h.a(id="captchaRefresh", title=_("refresh captcha")).action(self.init_captcha_image)
    return h.root
# ----------------------------------------------------------


class EmailRegistrationForm(editor.Editor):

    """Registration form for completing the email of a new external (and unconfirmed) user"""

    def __init__(self, app_title, app_banner, theme, username):
        self.email = editor.Property('').validate(validator.validate_email)
        self.email_repeat = editor.Property('').validate(validator.validate_email)
        self.header = component.Component(Header(app_title, app_banner, theme))
        self.error_message = u''
        self.username = username
        self.app_title = app_title

    def validate_emails_match(self):
        if self.email.value != self.email_repeat.value:
            self.email_repeat.error = _("The two emails don't match")

    def commit(self):

        properties = ('email', 'email_repeat')
        if not self.is_validated(properties):
            self.error_message = _(u'Unable to process. Check your input below.')
            return None

        u = usermanager.UserManager.get_by_username(self.username)
        if u:
            u.set_email_to_confirm(self.email.value)
            return self.username

        self.error_message = _(u'Something went wrong: user does not exist! Please contact the administrator of this site.')
        return None

    def on_ok(self, comp, application_url):
        username = self.commit()
        if username:
            comp.answer((username, application_url))


@presentation.render_for(EmailRegistrationForm)
def render_RegistrationForm(self, h, comp, *args):
    h << self.header.render(h, 'hide')
    with h.div(class_='regForm'):
        # autocomplete="off": do not store the password
        with h.form(autocomplete="off").post_action(self.validate_emails_match):
            h << h.p(_(u'Your profile is missing an email address. You have to enter a valid email address to open an account on %s') % self.app_title)
            with h.div(class_='fields'):
                fields = (
                    (_('Email address'), 'email', 'text', self.email),
                    (_('Email address (repeat)'), 'email', 'text', self.email_repeat),
                )
                for label, css_class, input_type, property in fields:
                    with h.div(class_='%s-field field' % css_class):
                        id_ = h.generate_id("field")
                        with h.label(for_=id_):
                            h << label
                        h << h.input(id=id_,
                                     type=input_type,
                                     value=property()).action(property).error(property.error)

            with h.div(class_='actions'):
                h << h.input(type='submit',
                             value=_("Create new account"),
                             class_="btn btn-primary").action(
                                self.on_ok, comp, h.request.application_url)

            h << _("Already have an account? ") << h.a(
                _("Log in")).action(comp.answer, (None, None))

    return h.root

# -----------------------------------------------------------------------


class RegistrationConfirmation(object):

    """Confirm a registration by sending a confirmation email, then acknowledge the success/failure
    of the operation"""

    def __init__(self, app_title, app_banner, theme):
        self.app_title = app_title
        self.header = component.Component(Header(app_title, app_banner, theme))


@presentation.render_for(RegistrationConfirmation, model='success')
def render_registration_confirmation_success(self, h, comp, *args):
    """Renders an registration acknowledgment message"""
    with h.body(class_='body-login'):
        h << self.header
        with h.div(class_='title'):
            h << h.h2(_(u'Sign up'))

        with h.div(class_='container'):
            with h.h3:
                h << _("Registration successful!")

            with h.p:
                h << _("""You are now a registered user of %s. You can login
                on the home page of the application.""") % self.app_title

            with h.form:
                with h.div(class_='actions'):
                    h << h.input(type='submit',
                                 class_="btn btn-primary",
                                 value=_("Ok")).action(comp.answer, h.request.application_url)

    return h.root


@presentation.render_for(RegistrationConfirmation, model='failure')
def render_registration_confirmation_failure(self, h, comp, *args):
    with h.body(class_='body-login'):
        h << self.header
        with h.div(class_='title'):
            h << h.h2(_(u'Sign up'))
            h << h.small(_("Registration failure!"), class_='error')
        with h.div(class_='container'):
            with h.p:
                h << _("""Registration failure! The token received is either expired or invalid. Please register again.""")

            with h.form:
                with h.div(class_='actions'):
                    h << h.input(type='submit', class_="btn btn-primary",
                                 value=_("Ok")).action(comp.answer, h.request.application_url)

    return h.root


# ----------------------------------------------------------

class PasswordEditor(editor.Editor):

    """Password editor, so that users can edit their password"""

    def __init__(self, app_title, app_banner, theme, get_user, check_old_password=True):
        self._get_user = get_user
        self.check_old_password = check_old_password
        self.old_password = editor.Property(
            '').validate(self.validate_old_password)
        self.password = editor.Property(
            '').validate(validator.validate_password)
        self.password_repeat = editor.Property(
            '').validate(validator.validate_password)
        self.header = component.Component(Header(app_title, app_banner, theme))
        self.error_message = u''

    def validate_old_password(self, value):
        validator.validate_non_empty_string(value)
        if not self.user.check_password(value):
            raise ValueError(_("Invalid password"))
        return value

    def validate_passwords_match(self):
        if self.password.value != self.password_repeat.value:
            self.password_repeat.error = _("The two passwords don't match")

    @property
    def user(self):
        return self._get_user()

    def commit(self):
        properties = ['password', 'password_repeat']
        if self.check_old_password:
            properties.insert(0, 'old_password')

        if not self.is_validated(properties):
            self.error_message = _(u'Invalid input!')
            return False

        # change the user's password in the database
        self.user.change_password(self.password.value)

        # change the current user credentials if applicable
        current_user = security.get_user()
        if current_user and (current_user.entity is self.user):
            current_user.update_password(self.password.value)

        return True


@presentation.render_for(PasswordEditor)
def render_password_editor(self, h, comp, *args):
    def commit():
        if self.commit():
            comp.answer(True)

    with h.body(class_='body-login'):
        h << self.header
        with h.div(class_='title'):
            h << h.h2(_(u'Change password'))
            if self.error_message:
                h << h.small(self.error_message, class_='error')
                self.error_message = u''
        with h.div(class_='container'):

            # autocomplete="off" prevent IE & Firefox from remembering the
            # passwords
            with h.form(autocomplete="off").post_action(self.validate_passwords_match):
                with h.div(class_='fields'):
                    fields = [
                        (_('Password'),
                            'password', 'password', self.password),
                        (_('Password (repeat)'), 'password-repeat',
                            'password', self.password_repeat)
                    ]
                    old_password_field = (_('Old Password'),
                                          'old-password', 'password', self.old_password)
                    if self.check_old_password:
                        fields.insert(0, old_password_field)

                    for label, css_class, input_type, property in fields:
                        with h.div(class_='%s-field field' % css_class):
                            id = h.generate_id("field")
                            with h.label(for_=id):
                                h << label
                            h << h.input(id=id,
                                         type=input_type,
                                         value=property()).action(property).error(property.error)

                with h.div(class_='actions'):
                    h << h.input(type='submit',
                                 value=_("Change password"),
                                 class_='btn btn-primary').action(commit)
                    h << u' '
                    h << h.input(type='submit',
                                 value=_("Cancel"),
                                 class_='btn').action(comp.answer)

        return h.root


# ----------------------------------------------------------

class PasswordResetForm(editor.Editor):

    """Password reset form, ask the user email"""

    def __init__(self, app_title, app_banner, theme, get_user_by_username):
        self._get_user_by_username = get_user_by_username
        self.username = editor.Property('').validate(self.validate_username)
        self.email = editor.Property('').validate(validator.validate_email)
        self.header = component.Component(Header(app_title, app_banner, theme))
        self.error_message = u''

    def validate_username(self, value):
        value = validator.validate_identifier(value)

        user = self._get_user_by_username(value)
        if not user:
            raise ValueError(
                _("This username is not registered on our database"))
        if user.source != 'application':
            raise ValueError(
                _("You can not change your password since you log in by an external service."))
        return value

    def validate_email_match_user_email(self):
        if self.username.error or self.email.error:  # there are errors already
            return

        user = self._get_user_by_username(self.username.value)
        if self.email() != user.email:
            self.email.error = _(
                "This email address does not match the user's email address")

    def commit(self):
        properties = ('username', 'email')
        if not self.is_validated(properties):
            self.error_message = _(u'Invalid input!')
            return None

        return self._get_user_by_username(self.username.value)


@presentation.render_for(PasswordResetForm)
def render_password_reset_form(self, h, comp, *args):
    application_url = h.request.application_url
    def commit():
        user = self.commit()
        if user:
            comp.answer((user.username, application_url))
    h << self.header.render(h, 'hide')
    with h.div(class_='regForm'):

        with h.p:
            h << _("""Please enter your username and your email address and you'll receive an email that contains a link to reset your password.""")

        with h.form.post_action(self.validate_email_match_user_email):
            with h.div(class_='fields'):
                fields = (
                    (_('Username'), 'username', 'text', self.username),
                    (_('Email address'), 'email', 'text', self.email),
                )
                for label, css_class, input_type, property in fields:
                    with h.div(class_='%s-field field' % css_class):
                        id = h.generate_id("field")
                        with h.label(for_=id):
                            h << label
                        h << h.input(id=id,
                                     type=input_type,
                                     value=property()).action(property).error(property.error)

            with h.div(class_='actions'):
                h << h.input(type='submit',
                             value=_("Reset password"),
                             class_='btn btn-primary').action(commit)

            h << (_("Remember your password?"), u' ', h.a(_("Log in")).action(comp.answer, (None, None)))

    return h.root


# ----------------------------------------------------------

class PasswordResetConfirmation(object):

    """Confirm a password reset by sending a confirmation email, then acknowledge the
    success/failure of the operation"""

    def __init__(self, app_title, app_banner, theme, get_user, confirmation_base_url):
        self.app_title = app_title
        self._get_user = get_user
        self.confirmation_base_url = confirmation_base_url
        self.token_generator = TokenGenerator(self._get_user().username, u'reset_password')
        self.header = component.Component(Header(self.app_title, app_banner, theme))
        self.alt_title = _(u'Reset password')

    @property
    def user(self):
        return self._get_user()

    @property
    def confirmation_url(self):
        token = self.token_generator.create_token().token
        return '/'.join((self.confirmation_base_url, 'reset', self._get_user().username, token))

    def confirm_password_reset(self, token):
        return self.token_generator.check_token(token)

    def reset_token(self, token):
        return self.token_generator.reset_token(token)

    def send_email(self, mail_sender):
        subst = dict(fullname=self.user.fullname,
                     app_title=self.app_title,
                     confirmation_url=self.confirmation_url)

        message = _("""
        Hello %(fullname)s,

        We received a request to reset your password. If you are at the origin of this
        request, please click on this confirmation link: %(confirmation_url)s. Otherwise,
        feel free to ignore this email.

        See you soon on %(app_title)s.
        """) % subst

        mail_sender.send(_("%(app_title)s: Password Reset") % subst,
                         [self.user.email],
                         textwrap.dedent(message).strip())


@presentation.render_for(PasswordResetConfirmation, model='success')
def render_password_reset_confirmation_failure(self, h, comp, *args):
    """Renders a password change acknowledgment message"""
    with h.body(class_='body-login'):
        h << self.header
        with h.div(class_='title'):
            h << h.h2(_(u'Change password'))

        with h.div(class_='container'):
            with h.h3:
                h << _("Password change successful!")

            with h.p:
                h << _("""Your password have been changed successfully.""")

            with h.form:
                with h.div(class_='actions'):
                    h << h.input(type='submit',
                                 class_='btn btn-primary',
                                 value=_("Ok")).action(comp.answer)
    return h.root


@presentation.render_for(PasswordResetConfirmation, model='email')
def render_password_reset_confirmation_failure(self, h, comp, *args):
    """Renders a password change acknowledgment message"""
    h << self.header.render(h, 'hide')
    with h.div(class_='regForm'):
        with h.h3:
            h << _("Email sent!")

        with h.p:
            h << _("""An email has been sent!""")

        with h.form:
            with h.div(class_='actions'):
                h << h.input(type='submit',
                             class_='btn btn-primary',
                             value=_("Ok")).action(comp.answer)
    return h.root


@presentation.render_for(PasswordResetConfirmation, model='failure')
def render_password_reset_confirmation_failure(self, h, comp, *args):
    with h.body(class_='body-login'):
        h << self.header
        with h.div(class_='title'):
            h << h.h2(_(u'Change password'))
            h << h.small(_("Password reset failure!"), class_='error')
        with h.div(class_='container'):
            with h.p:
                h << _("""Password reset failure! Please try again.""")

            with h.form:
                with h.div(class_='actions'):
                    h << h.input(type='submit',
                                 class_='btn btn-primary',
                                 value=_("Ok")).action(comp.answer)

    return h.root


# ----------------------------------------------------------

class EmailConfirmation(object):

    """Confirm a user email address by sending an email to the user with a confirmation link."""

    def __init__(self, app_title, app_banner, theme, get_user, confirmation_base_url='', moderator=''):
        self.app_title = app_title
        self._get_user = get_user
        self.moderator = moderator
        self.confirmation_base_url = confirmation_base_url
        self.token_generator = TokenGenerator(self._get_user().username, 'email_confirmation')
        self.header = component.Component(Header(app_title, app_banner, theme))
        self.alt_title = _('Sign up')

    @property
    def user(self):
        return self._get_user()

    def confirm_email_address(self, token):
        # too late, the user has been removed from the database
        if not self.user:
            return False

        # valid token?
        if not self.token_generator.check_token(token):
            return False

        self.user.confirm_email()

        return True

    def reset_token(self, token):
        return self.token_generator.reset_token(token)

    @property
    def confirmation_url(self):
        token = self.token_generator.create_token().token
        return '/'.join((self.confirmation_base_url, token))

    def send_email(self, mail_sender):
        '''
        If the email address ``true_dest`` is given, send the mail to
        that person instead of self.user.
        '''

        subst = dict(fullname=self.user.fullname,
                     login=self.user.username,
                     email=self.user.email_to_confirm,
                     app_title=self.app_title,
                     confirmation_url=self.confirmation_url)

        if self.moderator:
            message = _("""
            Hello moderator,

            %(fullname)s is willing to register an account (%(login)s) on the %(app_title)s application.

            Her (his) email address is %(email)s. Please verify it before accepting this account creation.

            In order to confirm that email address and accept the user's registration,
            please click on this confirmation link: %(confirmation_url)s.

            If you don't accept this request, feel free to ignore this email.
            Nothing will change into the user's pending account until your click
            on the confirmation link.

            Once confirmed, the user can login with her (his) login "%(login)s" and
            the password she (he) provided.
            """) % subst
            subject = _("%(app_title)s: Confirm user registration") % subst
        else:
            message = _("""
            Hello %(fullname)s,

            In order to confirm your email address in the %(app_title)s application,
            please click on this confirmation link: %(confirmation_url)s.

            If you are not at the origin of this request, feel free to ignore this
            email. Nothing will change into your account until your click on the
            confirmation link.

            Once confirmed, you can login with your login "%(login)s" and the password you just provided.

            See you soon on %(app_title)s.
            """) % subst
            subject = _("%(app_title)s: Confirm your email address") % subst

        mail_sender.send(subject,
                         [self.moderator if self.moderator else self.user.email_to_confirm],
                         textwrap.dedent(message).strip())


@presentation.render_for(EmailConfirmation)
def render_registration_confirmation(self, h, comp, *args):
    h << self.header.render(h, 'hide')
    with h.div(class_='regForm'):
        with h.h3:
            h << _("Registration request sent") if self.moderator else _("Confirm your email address!")

        with h.p:
            if self.moderator:
                h << _("""An email has been sent to the moderator. You will be contacted when
                your account is activated.""")
            else:
                h << _("""An email has been sent to your email address. You have to click on the
                confirmation link in this email in order to confirm your email address in the
                application.""")

        with h.form:
            with h.div(class_='actions'):
                h << h.input(type='submit',
                             class_='btn btn-primary',
                             value=_("Ok")).action(comp.answer)

    return h.root


# ----------------------------------------------------------

class EmailInvitation(object):

    """Send invitation email to the user with a confirmation link."""

    def __init__(self, app_title, app_banner, theme, email, host, board, confirmation_base_url):
        """ Initialization method

        In:
         - ``email`` -- email of the guest
         - ``home`` -- host (DataUser instance)
         - ``board`` -- target of invitation (DataBoard instance)
         - ``confirmation_base_url`` -- base url for confirmation link
        """
        self.app_title = app_title
        self.email = email
        self.confirmation_base_url = confirmation_base_url
        self.host = host
        self.board = board
        self.header = component.Component(Header(app_title, app_banner, theme))
        self.token_generator = TokenGenerator(email,
                                              u'invite board %s' % board.id, expiration_delay=timedelta(days=2))

    @property
    def confirmation_url(self):
        who = usermanager.UserManager.get_by_email(self.email)
        if who is None:
            token = self.token_generator.create_token()
            self.board.pending.append(token)
        else:
            self.board.add_member(who)
        return u'/'.join((self.confirmation_base_url, self.board.url))

    def send_email(self, mail_sender):
        subst = dict(app_title=self.app_title,
                     board_title=self.board.title,
                     email=self.email,
                     host_name=self.host.fullname,
                     host_email=self.host.email,
                     confirmation_url=self.confirmation_url)

        message = _("""
        Hello,

        You were invited by %(host_name)s (%(host_email)s) to the board "%(board_title)s".

        To view the board, click on this link : %(confirmation_url)s.

        If you don't already have an account, register with email %(email)s and you will automatically join the board.

        See you soon on %(app_title)s.
        """) % subst

        mail_sender.send(_('%(app_title)s: Invitation to the board "%(board_title)s"') % subst,
                         [self.email],
                         textwrap.dedent(message).strip())

# ----------------------------------------------------------


class RegistrationTask(component.Task):

    """A task that handles the user registration process"""

    def __init__(self, app_title, app_banner, theme, mail_sender_service, assets_manager_service,
                 moderator='', identicons=False, username=''):
        '''
        Register a new user (`username` not provided)
        or register email for an existing unconfirmed user (`username` provided).
        If `moderator` is set to an email address, all confirmation requests are sent there instead of user's email.
        '''
        self.app_title = app_title
        self.app_banner = app_banner
        self.theme = theme
        self.mail_sender = mail_sender_service
        self.assets_manager = assets_manager_service
        self.moderator = moderator
        self.identicons = identicons
        self.state = None  # task state, initialized by a URL rule
        self.user_manager = usermanager.UserManager()
        self.username = username
        self.alt_title = _(u'Sign up')

    def _create_email_confirmation(self, username, confirmation_base_url=''):
        confirmation_url = '/'.join((confirmation_base_url, 'register', username))
        get_user = lambda: usermanager.UserManager.get_by_username(username)
        return EmailConfirmation(self.app_title, self.app_banner, self.theme, get_user, confirmation_url, self.moderator)

    def go(self, comp):
        if not self.state:
            # step 1:
            # - ask the user to fill a registration form
            # - send him a confirmation email
            if self.username:
                username, application_url = comp.call(
                    EmailRegistrationForm(
                        self.app_title,
                        self.app_banner,
                        self.theme,
                        self.username
                    )
                )
            else:
                username, application_url = comp.call(RegistrationForm(
                    self.app_title, self.app_banner, self.theme))
            if username:
                if self.identicons:
                    appuser = self.user_manager.get_app_user(username)
                    appuser.reset_avatar(self.assets_manager)
                confirmation = self._create_email_confirmation(username, application_url)
                confirmation.send_email(self.mail_sender)
                comp.call(confirmation)
        else:
            # step 2: the user confirms his email by clicking on the
            # confirmation link
            # - check the token (to avoid cheating)
            # - show a confirmation screen
            username, token = self.state

            confirmation = self._create_email_confirmation(username)
            if confirmation.confirm_email_address(token):
                log.debug(_("Registration successful for user %s") % username)
                base_url = comp.call(RegistrationConfirmation(self.app_title, self.app_banner, self.theme), model='success')
            else:
                log.debug(_("Registration failure for user %s") % username)
                base_url = comp.call(RegistrationConfirmation(self.app_title, self.app_banner, self.theme), model='failure')

            redirect_to(base_url)

# ----------------------------------------------------------


class TokenGenerator(object):

    """Generate or check tokens"""

    def __init__(self, username, action, expiration_delay=UserConfirmationTimeout):
        """Create a token generator"""
        self.action = action
        self.username = username
        self.expiration_delay = expiration_delay

    def create_token(self):
        """Create a time-stamped token"""

        token = unicode(
            hashlib.sha512(str(time.time()) + self.username).hexdigest())

        DataToken.delete_by_username(self.username, self.action)
        # create new one
        token_instance = DataToken.new(token, self.username, self.action)
        return token_instance

    def get_tokens(self):
        query = DataToken.query.filter_by(username=self.username)
        query = query.filter(DataToken.action.like(self.action + '%'))
        return query

    def check_token(self, token):
        """Check that the token is valid"""
        t = DataToken.get(token)
        return t and datetime.now() <= (t.date + self.expiration_delay)

    def reset_token(self, token):
        DataToken.get(token).delete()

# ----------------------------------------------------------


class PasswordResetTask(component.Task):

    """A task that handles the password reset process"""

    def __init__(self, app_title, app_banner, theme, mail_sender_service):
        """Be careful! The confirmation URL *should* be rooted"""
        self.app_title = app_title
        self.app_banner = app_banner
        self.theme = theme
        self.mail_sender = mail_sender_service
        self.state = None  # task state, initialized by a URL rule
        self.user_manager = usermanager.UserManager()
        self.alt_title = _(u'Reset password')

    def _get_user(self, username):
        return usermanager.UserManager.get_by_username(username)

    def _create_password_reset_confirmation(self, username, confirmation_base_url):
        return PasswordResetConfirmation(self.app_title, self.app_banner, self.theme,
                                         lambda: self._get_user(username),
                                         confirmation_base_url)

    def go(self, comp):
        if not self.state:
            # step 1:
            # - ask the user email
            # - send him a confirmation email
            username, application_url = comp.call(PasswordResetForm(self.app_title,
                                                   self.app_banner,
                                                   self.theme,
                                                   self._get_user))
            if username:
                confirmation = self._create_password_reset_confirmation(
                    username, application_url)
                confirmation.send_email(self.mail_sender)
                comp.call(confirmation, model='email')
                redirect_to(application_url)
        else:
            # step 2: the user clicked on the confirmation link on his email
            # - check the token (to avoid cheating)
            # - ask for the new password
            username, token, application_url = self.state

            confirmation = self._create_password_reset_confirmation(username, application_url)
            if confirmation.confirm_password_reset(token):
                log.debug(_("Resetting the password for user %s") % username)
                ret = comp.call(PasswordEditor(self.app_title, self.app_banner, self.theme,
                                         lambda username=username: self._get_user(username),
                                         check_old_password=False))
                if ret:
                    confirmation.reset_token(token)
                    comp.call(confirmation, model='success')

            else:
                log.debug(_("Password reset failure for user %s") % username)
                comp.call(confirmation, model='failure')

            redirect_to(application_url)


@presentation.init_for(PasswordResetTask, "len(url) == 2")
def init_password_reset_task(self, url, comp, http_method, request):
    self.state = (url[0], url[1], request.application_url)


# ----------------------------------------------------------

class ChangeEmailConfirmation(object):

    def __init__(self, app_title, app_banner, theme, redirect_url='/'):
        self.redirect_url = redirect_url
        self.header = component.Component(Header(app_title, app_banner, theme))


@presentation.render_for(ChangeEmailConfirmation, 'success')
@presentation.render_for(ChangeEmailConfirmation, 'failure')
def render_change_email_confirmation_success(self, h, comp, model):
    """Renders an email change acknowledgment message"""
    with h.body(class_='body-login'):
        h << self.header
        with h.div(class_='title'):
            h << h.h2(_(u'Change email'))

        with h.div(class_='container'):
            with h.h3:
                if model == 'success':
                    h << _("Email change successful!")
                else:
                    h << _("Email change failure!")

            with h.p:
                if model == 'success':
                    h << _("""Your email have been changed successfully.""")
                else:
                    h << _("""Email change failure! Please retry.""")

            with h.form:
                with h.div(class_='actions'):
                    h << h.input(type='submit',
                                 class_='btn btn-primary',
                                 value=_("Ok")).action(lambda: redirect_to(self.redirect_url))

    return h.root






# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import component, presentation, ajax, var, security
from nagare.i18n import _

from kansha.toolbox import overlay

from .comp import Label, CardLabels


def color_style(self):
    return 'background-color:%s' % self.data.color


@presentation.render_for(Label)
def render_Label(self, h, comp, *args):
    """Render the label as a simple colored block (text in it)"""
    with h.span(class_='card-label', style=color_style(self)):
        h << h.span(self.data.title)
    return h.root


@presentation.render_for(Label, model='color')
@presentation.render_for(Label, model='inactive')
def render_Label_color(self, h, comp, model):
    """Render the label as a simple colored block"""
    return h.span(class_='card-label ' + model, style=color_style(self), title=self.get_title())


@presentation.render_for(Label, model='edit-color')
def render_Label_edit_color(self, h, comp, *args):
    """Edit the label color"""
    # If label changed reload columns
    h << component.Component(overlay.Overlay(lambda r: comp.render(r, model='color'),
                                             lambda r: comp.render(r,
                                                                   model='edit-color-overlay'),
                                             dynamic=False,
                                             title=_('Change color')))

    return h.root


@presentation.render_for(Label, model='edit-color-overlay')
def render_Label_edit_color_overlay(self, h, comp, *args):
    """Color chooser contained in the overlay body"""
    v = var.Var(self.data.color)
    i = h.generate_id()
    h << h.div(id=i, class_='label-color-picker clearfix')
    with h.form:
        h << h.input(type='hidden', value=v(), id='%s-hex-value' % i).action(v)
        h << h.button(_('Save'), class_='btn btn-primary').action(
            ajax.Update(action=lambda v=v: self.set_color(v())))
        h << ' '
        h << h.button(_('Cancel'), class_='btn').action(lambda: None)
    h << h.script("YAHOO.kansha.app.addColorPicker(%s)" % ajax.py2js(i))
    return h.root


@presentation.render_for(CardLabels, model='header')
def render_CardLabels_header(self, h, comp, *args):
    """Show labels inline (used in card summary view)"""
    if self.colors:
        with h.div(class_='inline-labels'):
            h << (component.Component(d, model='color')
                  for d in self.labels)
    return h.root


@presentation.render_for(CardLabels, model='list')
@presentation.render_for(CardLabels, model='list-edit')
def render_CardLabels_list(self, h, comp, model):
    """Show labels inline for card edit view"""
    with h.div(class_='editable-labels'):
        for label in self.get_available_labels():
            label_model = 'color' if label in self.labels else 'inactive'
            if model == 'list-edit':
                h << h.a(component.Component(label, label_model)).action(self.activate, label)
            elif label_model == 'color':
                h << component.Component(label, 'color')
    return h.root


@presentation.render_for(CardLabels)
def render_CardLabels(self, h, comp, *args):
    """Add or remove labels to card"""
    if security.has_permissions('edit', self.card):
        h << comp.render(h.AsyncRenderer(), model="list-edit")
    else:
        h << comp.render(h, model="list")
    return h.root



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

from nagare.i18n import _, _N, _L
from nagare import ajax, component, presentation, security, var

from .comp import BoardCardFilter


@presentation.render_for(BoardCardFilter, 'search_results')
def render_search_results(self, h, comp, *args):
    h << h.script('YAHOO.kansha.app.highlight_cards(%s);' % ajax.py2js(list(self.card_matches), h))
    h << comp.render(h, 'num_matches')
    self._changed = False
    return h.root


@presentation.render_for(BoardCardFilter, 'footer')
def render_search_results(self, h, comp, *args):
    if self._changed:
        h << comp.render(h, 'search_results')
        h << h.script('$("#search input").val("");')
    return h.root


@presentation.render_for(BoardCardFilter)
@presentation.render_for(BoardCardFilter, 'num_matches')
def render_num_matches(self, h, comp, *args):
    local_h = h.SyncRenderer()
    input_class = ''
    if self.card_matches:
        if None in self.card_matches:
            local_h << local_h.span(_(u'No matches'), class_='nomatches')
            input_class = 'nomatches'
        else:
            n = len(self.card_matches)
            local_h << (_N(u'%d match', u'%d matches', n) % n)
            input_class = 'highlight'
    else:
        local_h << u' '
    res = (local_h.root if isinstance(local_h.root, basestring) else
           local_h.root.write_htmlstring())
    h << h.script(u'$("#show_results").html(%s);' % ajax.py2js(res).decode('utf-8'))
    h << h.script('document.getElementById("search").className = "%s";' % input_class)
    return h.root


@presentation.render_for(BoardCardFilter, 'search_input')
def render_num_matches(self, h, comp, *args):
    h.head.javascript_url('js/debounce.js')
    h.head.javascript_url('js/search.js?v=2c')

    search_cb = h.input.action(ajax.Update(
        action=self.search,
        component_to_update='show_results',
        render=lambda renderer: comp.render(renderer, 'search_results')
    )).get('onchange').replace('this', 'elt')
    oninput = 'debounce(this, function(elt) { %s; }, 500)' % search_cb
    h << h.div(id_='show_results')
    if self.card_matches:
        if None in self.card_matches:
            klass = 'nomatches'
        else:
            klass = 'highlight'
    else:
        klass = ''
    with h.div(id='search', class_=klass):
        h << h.input(type='text', placeholder=_(u'search'),
                     value=self.last_search,
                     oninput=oninput)
        with h.span(class_='icon'):
            h << h.i(class_='icon-search search_icon',
                     style='display:none' if self.last_search else '')
            h << h.a(h.i(class_='icon-cancel-circle'), href='#', class_='search_close',
                     style='' if self.last_search else 'display: none')
    h << comp.render(h, 'num_matches')

    return h.root



"""new asset path to assets

Revision ID: 24be36b8c67
Revises: 1bd634091036
Create Date: 2015-10-23 15:43:39.398148

"""

import sqlalchemy as sa
from alembic import op


# revision identifiers, used by Alembic.
revision = '24be36b8c67'
down_revision = '1bd634091036'

users = sa.Table(
    'user',
    sa.MetaData(),
    sa.Column('username', sa.VARCHAR(255), primary_key=True),
    sa.Column('picture', sa.Text)
)


def fix_path(oldpath):
    if not oldpath:
        return oldpath
    return oldpath.replace('/assets/', '/services/assets_manager/')


def restore_path(newpath):
    if not newpath:
        return newpath
    return newpath.replace('/services/assets_manager/', '/assets/')


def upgrade():
    connection = op.get_bind()
    for u in connection.execute(users.select()):
        connection.execute(
            users.update().where(
                users.c.username == u.username
            ).values(
                picture=fix_path(u.picture)
            )
        )


def downgrade():
    connection = op.get_bind()
    for u in connection.execute(users.select()):
        connection.execute(
            users.update().where(
                users.c.username == u.username
            ).values(
                picture=restore_path(u.picture)
            )
        )



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

import datetime

from nagare.i18n import _
from nagare.namespaces.xhtml import absolute_url
from nagare import ajax, component, presentation, security

from kansha import VERSION
from kansha.user.usermanager import UserManager

from .comp import Kansha


def answer_on_menu(self, comp, user, v):
    """
    In:
        - ``self`` -- Kansha application
        - ``comp`` -- Component wrapped application
        - ``user`` -- current user
        - ``v`` -- value returned
    """
    if v is None:
        comp.answer(None)
    else:
        self._on_menu_entry('boards')


@presentation.render_for(Kansha, model='menu')
def render_kansha_menu(self, h, comp, *args):
    """Main menu part"""
    with h.div(class_='nav-menu', onclick='YAHOO.kansha.app.toggleMainMenu(this)'):
        with h.ul(class_='actions'):
            # If login, display logout button
            user = security.get_user()
            if user:
                # User menu
                self.user_menu.on_answer(lambda v: answer_on_menu(self, comp, user, v))
                h << self.user_menu.render(h, 'menu')
            else:
                h << h.li(h.a(_(u"Login"), href=h.request.application_url))

        # Tab part
        h << h.span(self.title.render(h.AsyncRenderer()), class_="menu-title", id='kansha-nav-menu')
    return h.root


@presentation.render_for(Kansha, model='tab')
def render_kansha_tab(self, h, comp, *args):
    user = security.get_user()
    if user is None:
        h << h.a(self.app_title)
    else:
        app_user = UserManager.get_app_user(user.username)
        h << h.a(component.Component(app_user, "avatar"), self.app_title)
    return h.root


@presentation.render_for(Kansha, model='yui-deps')
def render_kansha_yui_deps(self, h, comp, *args):
    """YUI CSS and JS dependencies"""
    for e in ('container', 'colorpicker', 'autocomplete', 'resize', 'imagecropper'):
        h.head.css_url(ajax.YUI_INTERNAL_PREFIX +
                       '/%(module)s/assets/skins'
                       '/sam/%(module)s.css' % dict(module=e))

    h.head.javascript_url(
        ajax.YUI_INTERNAL_PREFIX + '/yahoo-dom-event/yahoo-dom-event.js')
    minified = ('connection', 'animation', 'dragdrop', 'element', 'get',
                'json', 'container', 'slider', 'colorpicker', 'datasource',
                'autocomplete', 'selector', 'resize', 'imagecropper', 'selector')
    for e in minified:
        h.head.javascript_url(ajax.YUI_INTERNAL_PREFIX +
                              '/%(module)s/%(module)s-min.js' % dict(module=e))
    return h.root


@presentation.render_for(Kansha, model='resync')
def render_Kansha_resync(self, h, comp, model):
    with h.div(id="resync", style='display:none'):
            h << h.a(_(u'click here to resync'), id='resync-action').action(self.initialization)
    return h.root


@presentation.render_for(Kansha, model='oip')
def render_Kansha_oip(self, h, comp, model):
    with h.div(id="oip", style='display:none'):
        h << h.h2(_(u'Operation in progress'), class_='title')
        with h.div(class_='content'):
            h << h.p(h.img(src='img/ajax-loader.gif'), u' ', _(u'Please wait until operation completes.'))
    return h.root


@presentation.render_for(Kansha, 'home_menu')
def render_user_profile__menu(self, h, comp, *args):
    with h.div(class_='menu'):
        with h.ul:
            for id_, entry in self.home_menu.iteritems():
                with h.li:
                    with h.a.action(self._on_menu_entry, id_):
                        h << h.i(class_='icon icon-' + entry.icon)
                        h << h.span(entry.label)
                        if self.selected == id_:
                            h << {'class': 'active'}
    return h.root


@presentation.render_for(Kansha)
def render_kansha(self, h, comp, *args):
    """Main renderer"""

    favicon_url = absolute_url(self.favicon, h.head.static_url)
    h.head << h.head.link(rel="icon", type="image/x-icon", href=favicon_url)
    h.head << h.head.link(rel="shortcut icon", type="image/x-icon", href=favicon_url)

    h << comp.render(h, model='yui-deps')

    h.head << h.head.meta(
        name='viewport', content='width=device-width, initial-scale=1.0')

    h.head.css_url('css/knacss.css')
    h.head.css_url('css/themes/fonts.css?v=2c')
    h.head.css_url('css/themes/kansha.css?v=2c')
    h.head.css_url('css/themes/%s/kansha.css?v=2c' % self.theme)

    h.head.javascript_url('js/jquery-2.1.3.min.js')
    h.head.javascript_url('js/jquery-ui-1.11.2.custom/jquery-ui.js')

    h.head.javascript_url("js/jquery-linkify/jquery.linkify.min.js")

    h.head.javascript_url('js/dnd.js?v=2d')
    h.head.javascript_url('js/kansha.js?v=2c')
    h.head.javascript_url('js/autocomplete.js')

    h.head.javascript_url('js/wysihtml/dist/minified/wysihtml.min.js?v=2c')
    h.head.javascript_url('js/wysihtml/dist/minified/wysihtml.toolbar.min.js?v=2c')
    h.head.javascript_url('js/wysihtml/parser_rules/simple.js')

    if self.selected == 'board':
        with h.body(class_='yui-skin-sam'):
            h << h.div(id="mask")
            h << comp.render(h, model='resync')
            h << comp.render(h, model='oip')
            with h.div(id='application'):
                h << comp.render(h, model='menu')
                h << self.content.on_answer(self.handle_event)
    else:
        h.head << h.head.title(self.app_title)
        with h.body(class_='yui-skin-sam'):
            with h.div(class_='wrap'):
                with h.div(class_='container'):
                    h << h.div(id="mask")
                    h << comp.render(h, model='resync')
                    h << comp.render(h, model='oip')
                    with h.div(id='application'):
                        h << comp.render(h, model='menu')

                        with h.div(class_='home'), h.div(class_='grid-2'):
                            h << comp.render(h, 'home_menu')
                            with h.div(class_='boards'):
                                # answer only happens when a new board is created
                                h << self.content.on_answer(self.select_board).render(h.AsyncRenderer())
            with h.div(class_='credits'):
                with h.div(class_='container'):
                    h << h.a(self.app_title, href='http://www.kansha.org', target='_blank') << (u' v%s - \u00a9 ' % VERSION)
                    h << h.a('Net-ng', href='http://www.net-ng.com', target='_blank') << (' %d' % datetime.date.today().year)

    h.head.javascript_url('js/nagare.js')

    return h.root



# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

import json
import random

from peak.rules import when

from nagare.i18n import _
from nagare.security import common
from nagare import component, database, i18n, security

from kansha import title
from kansha.card import Card
from kansha.services.search import schema
from kansha.cardextension import CardExtension
from kansha.services.actionlog.messages import render_event

from .models import DataChecklist, DataChecklistItem


@when(common.Rules.has_permission, "user and perm == 'checklist' and isinstance(subject, Card)")
def has_permission_Card_checklist(self, user, perm, card):
    return security.has_permissions('edit', card)


@when(render_event, "action=='card_add_list'")
def render_event_card_add_list(action, data):
    return _(u'User %(author)s has added the checklist "%(list)s" to card "%(card)s"') % data


@when(render_event, "action=='card_delete_list'")
def render_event(action, data):
    return _(u'User %(author)s has deleted the checklist "%(list)s" from card "%(card)s"') % data


@when(render_event, "action=='card_listitem_done'")
def render_event(action, data):
    return _(u'User %(author)s has checked the item %(item)s from the checklist "%(list)s", on card "%(card)s"') % data


@when(render_event, "action=='card_listitem_undone'")
def render_event(action, data):
    return _(u'User %(author)s has unchecked the item %(item)s from the checklist "%(list)s", on card "%(card)s"') % data


class NewChecklistItem(object):
    def __init__(self, show_button):
        self.show_button = show_button
        self.focus = False

    def set_show_button(self, show_button):
        self.show_button = show_button
        if not show_button:
            self.focus = True


class ChecklistItem(object):

    def __init__(self, id_, action_log, data=None):
        self.id = id_
        self.action_log = action_log
        data = data if data is not None else self.data
        self.title = component.Component(
            title.EditableTitle(
                self.get_title,
                placeholder=i18n._(u'Enter task')
            )
        ).on_answer(self.set_title)
        self.done = data.done

    @property
    def data(self):
        return DataChecklistItem.get(self.id)

    @property
    def index(self):
        return self.data.index

    @property
    def checklist_id(self):
        return self.data.checklist.id

    def get_title(self):
        return self.data.title

    def set_title(self, title):
        self.data.title = title

    def set_done(self):
        '''toggle done status'''
        self.data.done = self.done = not self.done
        item = self.data
        data = {'item': self.get_title(),
                'list': item.checklist.title,
                'card': item.checklist.card.title}
        self.action_log.add_history(
            security.get_user(),
            u'card_listitem_done' if self.done else u'card_listitem_undone',
            data)


class Checklist(object):
    def __init__(self, id_, action_log, data=None):
        self.id = id_
        self.action_log = action_log
        data = data if data is not None else self.data
        self.items = [component.Component(ChecklistItem(item.id, action_log, item)) for item in data.items]

        self.title = component.Component(
            title.EditableTitle(
                self.get_title,
                placeholder=i18n._(u'Add title')
            )
        ).on_answer(self.set_title)
        self.new_item = component.Component(NewChecklistItem(len(self.items))).on_answer(self.add_item_from_str)

    def update(self, other):
        self.data.update(other.data)
        self.items = [component.Component(ChecklistItem(item.id, self.action_log, item)) for item in self.data.items]

    def add_item_from_str(self, text):
        if text is None or not text.strip():
            return
        data_item = self.data.add_item_from_str(text)
        item = ChecklistItem(data_item.id, self.action_log, data_item)
        self.add_item(item)

    def add_item(self, item):
        item = component.Component(item)
        self.items.append(item)
        # UI specific
        if self.new_item:
            self.new_item().focus = True

    def remove_item(self, item):
        self.items.pop(item.index)
        self.data.remove_item(item.data)

    def delete_item(self, item):
        self.delete_index(item.index)

    def insert_item(self, index, item):
        self.data.insert_item(index, item.data)
        item = component.Component(item)
        self.items.insert(index, item)

    def delete_index(self, index):
        item = self.items.pop(index)
        self.data.delete_item(item().data)

    def get_title(self):
        return self.data.title

    def set_title(self, title):
        if not self.data.title:
            self.new_title(title)
        self.data.title = title

    def set_index(self, index):
        self.data.index = index
        return self

    def delete(self):
        self.items = []
        self.data.purge()
        self.data.delete()

    @property
    def total_items(self):
        return len(self.items)

    @property
    def total_items_done(self):
        return len([item for item in self.items if item().done])

    @property
    def progress(self):
        if not self.items:
            return 0
        return self.total_items_done * 100 / self.total_items

    @property
    def data(self):
        return DataChecklist.get(self.id)

    def new_title(self, title):
        cl = self.data
        data = {'list': title, 'card': cl.card.title}
        self.action_log.add_history(
                                  security.get_user(),
                                  u'card_add_list',
                                  data)


class Checklists(CardExtension):

    LOAD_PRIORITY = 30

    def __init__(self, card, action_log, configurator):
        super(Checklists, self).__init__(card, action_log, configurator)
        self.ck_cache = {}
        self.checklists = []

    def load_children(self):
        if not self.checklists:
            cklists = [(clist.id, Checklist(clist.id, self.action_log, clist)) for clist in self.data]
            self.ck_cache = dict(cklists)
            self.checklists = [component.Component(clist) for __, clist in cklists]

    @staticmethod
    def get_schema_def():
        return schema.Text(u'checklists')

    def update_document(self, document):
        document.checklists = u'\n'.join(cl.to_indexable() for cl in self.data)

    @property
    def data(self):
        return DataChecklist.get_by_card(self.card.data)

    def update(self, other):
        other.load_children()
        for index, checklist in enumerate(other.checklists):
            checklist = checklist()
            new_checklist = self.add_checklist()
            new_checklist.update(checklist)

    @property
    def total_items(self):
        return DataChecklist.total_items(self.card.data)

    @property
    def total_items_done(self):
        return DataChecklist.total_items_done(self.card.data)

    def delete_checklist(self, index):
        cl = self.checklists.pop(index)()
        del self.ck_cache[cl.id]
        for i in range(index, len(self.checklists)):
            self.checklists[i]().set_index(i)
        data = {'list': cl.get_title(), 'card': self.card.get_title()}
        cl.delete()
        if data['list']:
            self.action_log.add_history(
                                      security.get_user(),
                                      u'card_delete_list',
                                      data)

    def delete(self):
        self.load_children()
        for checklist in self.ck_cache.values():
            checklist.delete()

    def add_checklist(self):
        clist = DataChecklist(card=self.card.data)
        database.session.flush()
        ck = Checklist(clist.id, self.action_log, clist)
        self.ck_cache[clist.id] = ck
        ck.set_index(len(self.checklists))
        self.checklists.append(component.Component(ck))
        return ck

    def reorder(self, request, _resp):
        """Reorder checklists
        In:
         - ``ids`` -- checklist ids
        """
        ids = request.GET['data']
        cl_ids = map(lambda x: int(x.split('_')[-1]), json.loads(ids))
        self.checklists = [component.Component(self.ck_cache[cid].set_index(i))
                           for i, cid in enumerate(cl_ids)]

    def reorder_items(self, request, _resp):
        data = json.loads(request.GET['data'])
        item_id = int(data['id'].split('_')[-1])
        checklist_id = int(data['target'].split('_')[-1])
        item = ChecklistItem(item_id, self.action_log)
        source = self.ck_cache[item.checklist_id]
        checklist = self.ck_cache[checklist_id]
        source.remove_item(item)
        checklist.insert_item(data['index'], item)



#--
# Copyright (c) 2008-2013 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

#--
# Copyright (c) 2012-2015 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--


"""
Create the demo users.
Registered as a nagare-admin command.
Usage :
nagare-admin create-demo <app name | config file>
"""
import pkg_resources

from nagare import database
from nagare.admin import util, command

from kansha.user.usermanager import UserManager


# FIXME: too low level
def create_demo(app):
    # demo users
    user_manager = UserManager()
    assets_manager = app.assets_manager
    for i in xrange(1, 4):
        email = u'user%d@net-ng.com' % i
        username = u'user%d' % i
        user = user_manager.create_user(username, u'password', u'user %d' % i, email)
        user.confirm_email()
        appuser = user_manager.get_app_user(username, user)
        appuser.reset_avatar(assets_manager)
        database.session.flush()
    # demo templates
    # TODO


class CreateDemo(command.Command):

    desc = 'Create demo users and contents.'

    @staticmethod
    def set_options(optparser):
        optparser.usage += ' [application]'

    @staticmethod
    def run(parser, options, args):

        try:
            application = args[0]
        except IndexError:
            application = 'kansha'

        (cfgfile, app, dist, conf) = util.read_application(application,
                                                           parser.error)
        requirement = (
            None if not dist
            else pkg_resources.Requirement.parse(dist.project_name)
        )
        data_path = (
            None if not requirement
            else pkg_resources.resource_filename(requirement, '/data')
        )

        (active_app, databases) = util.activate_WSGIApp(
            app, cfgfile, conf, parser.error, data_path=data_path)
        for (database_settings, populate) in databases:
            database.set_metadata(*database_settings)
        if active_app:
            create_demo(active_app)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from kansha.cardextension.tests import CardExtensionTestCase

from .comp import CardLabels


class CardLabelsTest(CardExtensionTestCase):

    extension_name = 'labels'
    extension_class = CardLabels

    def test_activate(self):
        self.assertTrue(self.extension.get_available_labels() > 0)
        self.assertEqual(len(self.extension.labels), 0)
        label = self.extension.get_available_labels()[0]
        self.extension.activate(label)
        self.assertIn(label, self.extension.labels)
        self.extension.activate(label)
        self.assertNotIn(label, self.extension.labels)

    def test_copy(self):
        labels = self.extension.get_available_labels()
        for label in labels:
            self.extension.activate(label)
        cpy = self.extension_copy
        labels2 = zip(self.extension.labels, cpy.labels)
        for labela, labelb in labels2:
            assert(labela.get_title() == labelb.get_title())

    def test_update_document(self):
        doc = self.card.schema(docid=None)
        label = self.extension.get_available_labels()[0]
        self.extension.activate(label)
        label = self.extension.get_available_labels()[1]
        self.extension.activate(label)
        self.extension.update_document(doc)
        self.assertEqual(doc.labels, u'Green Red')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import imghdr

import peak.rules
from nagare.i18n import _, _L
from nagare import editor, presentation, security

from kansha import validator
from kansha.authentication.database import forms as registation_forms

from .usermanager import UserManager


LANGUAGES = {'en': _L('english'),
             'fr': _L('french'),
             'ja': _L('japanese'),
             'tr': _L('turkish')}


class BasicUserForm(editor.Editor):

    """ Basic User Form

    View all fields but you can modify language only
    """

    fields = {'username', 'email', 'fullname', 'language', 'picture'}

    def __init__(self, app_title, app_banner, theme, target, *args):
        """
        In:
         - ``target`` -- DataUser instance
        """
        self.theme = theme
        super(BasicUserForm, self).__init__(target, self.fields)

    def commit(self):
        super(BasicUserForm, self).commit(self.fields)

    @property
    def target(self):
        if not self._target:
            self._target = self._get_target()
        return self._target

    @target.setter
    def target(self, value):
        self._target = value
        self._get_target = lambda cls = value.__class__, id_ = value._sa_instance_state.key[
            1]: cls.get(id_)

    def __getstate__(self):
        state = self.__dict__
        state['_target'] = None
        return state


@presentation.render_for(BasicUserForm)
def render(self, h, comp, *args):
    return comp.render(h.SyncRenderer(), 'edit')


@presentation.render_for(BasicUserForm, model='edit')
def render(self, h, comp, *args):
    h.head.css_url('css/themes/home.css?v=2c')
    h.head.css_url('css/themes/%s/home.css?v=2c' % self.theme)

    with h.div:
        with h.form:
            with h.ul:
                with h.li:
                    h << (h.label(_('Username')),
                          h.input(type='text', disabled=True, value=self.username()))
                with h.li:
                    h << (h.label(_('Fullname')),
                          h.input(type='text', disabled=True, value=self.fullname()))
                with h.li:
                    h << (h.label(_('Email')),
                          h.input(type='text', disabled=True, value=self.email()))
                with h.li:
                    h << (h.label(_('Language')))
                    with h.select().action(self.language).error(self.language.error):
                        for (id, lang) in LANGUAGES.items():
                            if self.language() == id:
                                h << h.option(_(lang), value=id, selected='')
                            else:
                                h << h.option(_(lang), value=id)
                if self.target.picture:
                    with h.li:
                        h << (h.label(_('Picture')))
                        h << h.div(
                            h.img(src=self.target.picture, class_='avatar big'))

            with h.div:
                h << h.input(value=_('Save'), class_='btn btn-primary',
                             type='submit').action(self.commit)
    return h.root


def get_userform(app_title, app_banner, theme, source):
    """ Default method to get UserForm Class

    In:
     - ``source`` -- source of user (application, google...)
    """
    return BasicUserForm


class UserForm(BasicUserForm):

    def __init__(self, app_title, app_banner, theme, target, assets_manager_service, mail_sender_service):
        """
        In:
         - ``target`` -- DataUser instance
         - ``mail_sender_service`` -- MailSender service
        """
        super(UserForm, self).__init__(app_title, app_banner, theme, target, self.fields)

        self.app_title = app_title
        self.app_banner = app_banner
        self.theme = theme
        self.mail_sender = mail_sender_service
        self.assets_manager = assets_manager_service
        self.username.validate(self.validate_username)
        self.fullname.validate(validator.validate_non_empty_string)

        # Add other properties (email to confirm, passwords...)
        self.email_to_confirm = editor.Property(
            target.email).validate(validator.validate_email)
        self.old_password = editor.Property(
            '').validate(self.validate_old_password)
        self.password = editor.Property('').validate(self.validate_password)
        self.password_repeat = editor.Property(
            '').validate(self.validate_passwords_match)
        self.user_manager = UserManager()

    def validate_username(self, value):
        """Check username

        In:
         - ``value`` -- username (must be unique)
        Return:
         - username value if username is unique (else raise an exception)
        """
        value = validator.validate_identifier(value)
        # check that this user name does not exist
        user = self.user_manager.get_by_username(value)
        if user:
            raise ValueError(_('Username %s is not available. Please choose another one.')
                             % value)
        return value

    def validate_password(self, value):
        """Check new password

        In:
         - ``value`` -- new password (must be greater than 6 chars)
        Return:
         - password value if password is ok (else raise an exception)
        """
        # check password complexity
        min_len = 6

        if len(value) < min_len and len(value) > 0:
            raise ValueError(_('Password too short: should have at least %d characters')
                             % min_len)
        return value

    def validate_old_password(self, value):
        """Check old password

        In:
         - ``value`` -- old password
        Return:
         - password value if value is the old password
        """
        if len(value) == 0 or security.get_user().data.check_password(value):
            return self.validate_password(value)
        raise ValueError(_('''This password doesn't match the old one.'''))

    def validate_passwords_match(self, value):
        """Check if confirmation password and password are equals

        In:
         - ``value`` -- confirmation password
        Return:
         - password value if confirmation password and password are equals
        """
        if self.password.value == value:
            return value
        else:
            raise ValueError(_('''The two passwords don't match'''))

    def _create_email_confirmation(self, application_url):
        """Create email confirmation"""
        confirmation_url = '/'.join(
            (application_url, 'new_mail', self.username()))
        return registation_forms.EmailConfirmation(self.app_title, self.app_banner, self.theme, lambda: security.get_user().data, confirmation_url)

    def commit(self, application_url):
        """ Commit method

        If email changes, send confirmation mail to user
        If password changes, check passwords rules
        """
        if not self.is_validated(self.fields):
            return
        # Test if email_to_confirm has changed
        if (self.target.email_to_confirm != self.email_to_confirm() and
                self.target.email != self.email_to_confirm()):
            # Change target email_to_confirm (need it to send mail)
            self.target.email_to_confirm = self.email_to_confirm()
            confirmation = self._create_email_confirmation(application_url)
            confirmation.send_email(self.mail_sender)
            self.email_to_confirm.info = _(
                'A confirmation email has been sent.')
        # Test if password has changed
        if (len(self.password()) > 0 and
                not(self.old_password.error) and
                not(self.password_repeat.error)):
            user = security.get_user()
            user.data.change_password(self.password())
            user.update_password(self.password())
            self.password_repeat.info = _('The password has been changed')
        super(BasicUserForm, self).commit(self.fields)

    def set_picture(self, new_file):
        uid = self.target.username
        self.picture.error = None
        error = None

        # No value, exit
        if new_file == '':
            return None

        try:
            validator.validate_file(
                new_file, self.assets_manager.max_size, _(u'File must be less than %d KB'))
        except ValueError, e:
            error = e.message
        if imghdr.what(new_file.file) is None:
            error = _(u'Invalid image file')

        if error:
            self.picture.error = error
            return None

        # Remove old value
        if self.target.picture:
            self.assets_manager.delete(uid)

        # Save new value
        self.assets_manager.save(new_file.file.read(), file_id=uid, metadata={
            'filename': new_file.filename, 'content-type': new_file.type}, THUMB_SIZE=(100, 100))
        self.picture(self.assets_manager.get_image_url(uid, size='thumb'))


@presentation.render_for(UserForm, 'edit')
def render(self, h, comp, *args):
    h.head.css_url('css/themes/home.css?v=2c')
    h.head.css_url('css/themes/%s/home.css?v=2c' % self.theme)

    with h.div:
        with h.form:
            with h.ul:
                with h.li:
                    h << (h.label(_('Username')),
                          h.input(type='text', readonly=True, value=self.username()))
                with h.li:
                    h << (h.label(_('Fullname')),
                          h.input(type='text', value=self.fullname()).action(self.fullname))
                with h.li:
                    h << (h.label(_('Email')),
                          h.input(type='text', value=self.email_to_confirm()).action(self.email_to_confirm).error(self.email_to_confirm.error))
                    if hasattr(self.email_to_confirm, 'info'):
                        h << h.span(
                            self.email_to_confirm.info, class_='info-message')
                with h.li(h.label(_('Language'))):
                    with h.select().action(self.language).error(self.language.error):
                        for (id, lang) in LANGUAGES.items():
                            if self.language() == id:
                                h << h.option(lang, value=id, selected='')
                            else:
                                h << h.option(lang, value=id)

                with h.li:
                    h << h.label(_('Picture'))
                    picture = self.target.get_picture()
                    if picture:
                        with h.label(for_='picture'):
                            h << h.img(
                                src=self.target.get_picture(), class_='avatar big')
                    h << h.input(type='file', id='picture').action(self.set_picture)\
                        .error(self.picture.error)

                with h.li:
                    h << (h.label(_('Old Password')),
                          # FF hack, don't save my password please
                          h.input(type='password', style='display:none'),
                          h.input(type='password').action(self.old_password)
                          .error(self.old_password.error))
                    if hasattr(self.password_repeat, 'info'):
                        h << h.span(
                            self.password_repeat.info, class_='info-message')
                with h.li:
                    h << (h.label(_('New password')),
                          h.input(type='password').action(self.password).error(self.password.error))
                with h.li:
                    h << (h.label(_('Repeat new password')),
                          h.input(type='password').action(self.password_repeat)
                          .error(self.password_repeat.error))

            with h.div(class_=''):
                h << h.input(_('Save'),
                             class_='btn btn-primary',
                             type='submit').action(self.commit, h.request.application_url)
    return h.root


@peak.rules.when(get_userform, """source == 'application'""")
def get_userform(app_title, app_banner, theme, source):
    """ User form for application user
    """
    def factory_compatible_with_services(target, services_service):
        return services_service(UserForm, app_title, app_banner, theme, target)

    return factory_compatible_with_services


class ExternalUserForm(BasicUserForm):
    pass


@peak.rules.when(get_userform, """source != 'application'""")
def get_userform(app_title, app_banner, theme, source):
    """ User form for application user
    """
    def factory_compatible_with_services(target, services_service):
        return services_service(ExternalUserForm, app_title, app_banner, theme, target)

    return factory_compatible_with_services



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--
from paste import fileapp
from webob.exc import WSGIHTTPException

from nagare import presentation

from kansha.exceptions import NotFound
from .services_repository import Service


class AssetsManager(Service):
    LOAD_PRIORITY = 10
    CONFIG_SPEC = {
            'basedir': 'string',
            'max_size': 'integer(default=2048)',   # Max file size in kilobytes
            'baseurl': 'string'
        }

    THUMB_SIZE = (68, 68)
    MEDIUM_WIDTH = 425
    COVER_SIZE = (MEDIUM_WIDTH, 250)

    def copy(self, file_id):
        '''Copy a file from its file_id

        In:
            - ``file_id`` -- file id of the source file
        Return:
            - copied file id'''

    def save(self, data, file_id=None, metadata={}):
        """Save data, metadata and return an id

        In:
            - ``data`` -- data file
            - ``file_id`` -- file id, if None create an random id
            - ``metadata`` -- metdata of file (dict format)
        Return:
            - the file_id
        """
        pass

    def delete(self, file_id):
        '''Delete a file'''
        pass

    def load(self, file_id):
        """return data and metadata

        In:
            - ``file_id`` -- file identifier
        Return:
            - a tuple (data, metadata)
        """
        pass

    def update_metadata(self, file_id, metadata):
        """Update metadata of the file

        In:
            - ``file_id`` -- file identifier
            - ``metadata`` -- new metadata
        """
        pass

    def get_metadata(self, file_id):
        """Return metadata of the file

        Return:
            - metadata (dict format)
        """
        pass

    def data_for_src(self, file_id):
        """Return data for attribute src of img tag

        In:
            - ``file_id`` -- file identifier
        Return:
            - src attribute of image tag
        """
        data, metadata = self.load(file_id, True)
        return "data:image/gif;base64,%s" % data.encode('base64')


@presentation.init_for(AssetsManager, "len(url) >= 2")
def init_assets(self, url, comp, http_method, request):
    try:
        metadata = self.get_metadata(url[0])
    except IOError:
        raise NotFound()

    content_type = metadata.get('content-type')
    headers = {'content-type': content_type} if content_type else {}

    raise FileResponse(self._get_filename(url[0],
                                          url[1] if len(url) > 1 else None),
                       **headers)


class FileResponse(WSGIHTTPException):

    def __init__(self, file_path, **headers):
        self._file_path = file_path
        self._headers = headers

    def __call__(self, environ, start_response):
        app = fileapp.FileApp(self._file_path,
                              allowed_methods=('GET', ),
                              cache_control='private, must-revalidate',
                              **self._headers)
        return app(environ, start_response)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from kansha.cardextension.tests import CardExtensionTestCase

from .comp import CardWeightEditor


class CardWeightTest(CardExtensionTestCase):

    extension_name = 'weight'
    extension_class = CardWeightEditor

    def test_copy(self):
        self.extension.weight(u'25')
        self.extension.commit()
        self.assertEqual(self.extension.data.weight, 25)
        cpy = self.extension_copy
        self.assertEqual(self.extension.weight(), cpy.weight())



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from sqlalchemy import func
from elixir import ManyToOne
from elixir import using_options
from elixir import Field, Integer, Unicode
from nagare.database import session


from kansha.models import Entity
from kansha.card.models import DataCard
from kansha.column.models import DataColumn


class DataBoardWeightConfig(Entity):
    using_options(tablename='board_weight_config')

    board = ManyToOne('DataBoard', ondelete='cascade')

    weighting_cards = Field(Integer, default=0)
    weights = Field(Unicode(255), default=u'0, 1, 2, 3, 5, 8, 13')

    def reset_card_weights(self):
        q = session.query(DataCardWeight).join(DataCard).join(DataColumn)
        for cw in q.filter(DataColumn.board == self.board):
            cw.weight = 0

    def total_weight(self):
        q = session.query(func.sum(DataCardWeight.weight)).join(DataCard).join(DataColumn)
        return q.filter(DataColumn.board == self.board).scalar()


class DataCardWeight(Entity):
    using_options(tablename='card_weight')

    weight = Field(Integer, default=0)
    card = ManyToOne('DataCard', ondelete='cascade')

    @classmethod
    def new(cls, card):
        """Create and persist"""
        weight = cls(card=card)
        session.add(weight)
        session.flush()
        return weight

    def update(self, other):
        self.weight = other.weight
        session.flush()

    @classmethod
    def get_by_card(cls, card):
        q = cls.query
        q = q.filter_by(card=card)
        return q.first()



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from sqlalchemy import func
from elixir import ManyToOne
from elixir import using_options

from nagare.database import session

from kansha.models import Entity


class DataVote(Entity):
    using_options(tablename='vote')
    user = ManyToOne('DataUser', ondelete='CASCADE')
    card = ManyToOne('DataCard', ondelete='CASCADE')

    @classmethod
    def new(cls, card, user):
        """Create and persist."""
        vote = cls(card=card, user=user)
        session.add(vote)
        session.flush()
        return vote

    @classmethod
    def get_vote(cls, card, user):
        '''Return Vote instance which match with user and card

        In:
            - ``card`` -- DataCard instance
            - ``user`` -- DataUser instance
        Return:
            - DataVote instance
        '''
        return cls.get_by(user=user, card=card)

    @classmethod
    def has_voted(cls, card, user):
        '''Return if a user has voted for a given card

        In:
            - ``card`` -- DataCard instance
            - ``user`` -- DataUser instance
        Return:
            - True if a vote is found, False otherwise
        '''
        return cls.get_vote(card, user) is not None

    @classmethod
    def count_votes(cls, card):
        q = cls.query
        q = q.filter(cls.card == card)
        return q.with_entities(func.count()).scalar()

    @classmethod
    def purge(cls, card):
        for vote in cls.query.filter_by(card=card):
            vote.delete()



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from kansha.cardextension.tests import CardExtensionTestCase

from .comp import CardDescription


class CardDescriptionTest(CardExtensionTestCase):

    extension_name = 'description'
    extension_class = CardDescription

    def test_change_desc(self):
        self.extension.text = u'test'
        self.assertEqual(self.extension.text, u'test')
        self.extension.change_text(u'test2')
        self.assertEqual(self.extension.text, u'<p>test2</p>')
        self.assertEqual(self.extension.text, u'<p>test2</p>')

    def test_copy(self):
        self.extension.text = u'test'
        cpy = self.extension_copy
        self.assertEqual(cpy.text, u'test')

    def test_update_document(self):
        doc = self.card.schema(docid=None)
        self.extension.text = u'some words'
        self.extension.update_document(doc)
        self.assertEqual(doc.description, u'some words')
        self.extension.text = u'<b><i>so</i>me</b> <u>HTML</u>'
        self.extension.update_document(doc)
        self.assertEqual(doc.description, u'some HTML')
        self.assertEqual(doc.description, u'some HTML')
        self.extension.text = u'éàü'
        self.extension.update_document(doc)
        self.assertEqual(doc.description, u'éàü')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from collections import OrderedDict

from nagare import security, component, i18n

from kansha import events

from .comp import Board, BOARD_PRIVATE, BOARD_PUBLIC


class BoardsManager(object):
    def __init__(self, app_title, app_banner, theme, card_extensions, search_engine_service, services_service):
        self.app_title = app_title
        self.app_banner = app_banner
        self.theme = theme
        self.card_extensions = card_extensions
        self.search_engine = search_engine_service
        self._services = services_service

        self.last_modified_boards = []
        self.my_boards = []
        self.guest_boards = []
        self.shared_boards = []
        self.archived_boards = []
        self.templates = {}

    def get_by_id(self, id_):
        board = None
        if Board.exists(id=id_):
            return self._services(Board, id_, self.app_title, self.app_banner, self.theme,
                                  self.card_extensions)
        return board

    def get_by_uri(self, uri):
        board = None
        if Board.exists(uri=uri):
            id_ = Board.get_id_by_uri(uri)
            return self._services(Board, id_, self.app_title, self.app_banner, self.theme,
                                  self.card_extensions)
        return board

    def create_board_from_template(self, template_id, user=None):
        if user is None:
            user = security.get_user()
        template = self._services(
            Board, template_id, self.app_title, self.app_banner, self.theme,
            self.card_extensions)
        new_board = template.copy(user)
        new_board.archive_column = new_board.create_column(index=-1, title=i18n._(u'Archive'))
        new_board.archive_column.is_archive = True
        new_board.mark_as_template(False)
        return new_board

    def create_template_from_board(self, board, title, description, shared, user=None):
        if user is None:
            user = security.get_user()
        template = board.copy(user)
        template.mark_as_template()
        template.set_title(title)
        template.set_description(description)
        template.set_visibility(BOARD_PRIVATE if not shared else BOARD_PUBLIC)
        return template

    def load_user_boards(self):
        user = security.get_user()
        self.my_boards = []
        self.guest_boards = []
        self.archived_boards = []
        last_modifications = set()
        for board_obj in self._services(Board.get_all_boards, user, self.app_title,
                                        self.app_banner, self.theme,
                                        self.card_extensions,
                                        load_children=False):
            if (security.has_permissions('manage', board_obj) or
                    security.has_permissions('edit', board_obj)):
                board_comp = component.Component(board_obj)
                if board_obj.archived:
                    self.archived_boards.append(board_comp)
                else:
                    last_activity = board_obj.get_last_activity()
                    if last_activity is not None:
                        last_modifications.add((last_activity, board_comp))
                    if security.has_permissions('manage', board_obj):
                        self.my_boards.append(board_comp)
                    elif security.has_permissions('edit', board_obj):
                        self.guest_boards.append(board_comp)

        last_5 = sorted(last_modifications, reverse=True)[:5]
        self.last_modified_boards = [comp for _modified, comp in last_5]

        self.shared_boards = [
            component.Component(board_obj) for board_obj in
            self._services(Board.get_shared_boards, self.app_title,
                           self.app_banner, self.theme,
                           self.card_extensions,
                           load_children=False)
        ]

        public, private = Board.get_templates_for(user)
        self.templates = {'public': [(b.id, b.template_title) for b in public],
                          'private': [(b.id, b.template_title) for b in private]}

    def purge_archived_boards(self):
        for board in self.archived_boards:
            board().load_children()
            board().delete()
        self.load_user_boards()

    def handle_event(self, event):
        if event.is_kind_of(events.BoardAccessChanged):
            if event.is_(events.BoardDeleted):
                board = event.emitter
                board.load_children()
                board.delete()
            return self.load_user_boards()

    #####

    def create_board(self, board_id, comp):
        """Create a new board from template for current user."""
        new_board = self.create_board_from_template(board_id)
        comp.answer(new_board.id)




# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

from nagare.i18n import _
from nagare import presentation, ajax, security, component

from .comp import Gallery, Asset, AssetCropper


def render_image(self, h, comp, size, randomize=False, **kw):
    metadata = self.assets_manager.get_metadata(self.filename)
    src = self.assets_manager.get_image_url(self.filename, size)
    if randomize:
        src += '?r=' + h.generate_id()
    return h.img(title=metadata['filename'], alt=metadata['filename'],
                 src=src, **kw)


def render_file(self, h, comp, size, **kw):
    kw['class'] += ' file_icon'
    metadata = self.assets_manager.get_metadata(self.filename)
    res = [h.img(title=metadata['filename'], alt=metadata['filename'],
                 src="img/file-icon.jpg", **kw)]
    if size == 'medium':
        res.append(h.span(metadata['filename']))
    return res


CONTENT_TYPES = {'image/png': render_image,
                 'image/jpeg': render_image,
                 'image/pjpeg': render_image,
                 'image/gif': render_image}


@presentation.render_for(Gallery)
def render(self, h, comp, *args):
    self.load_assets()
    with h.div(id='gal' + self.comp_id):
        with h.div(class_='nbItems'):
            h << comp.render(h, model='badge')
        with h.div(id="card-gallery"):
            h << comp.render(h, self.model)
    return h.root


@presentation.render_for(Gallery, 'view')
def render_Gallery_view(self, h, comp, model):
    model = 'edit' if security.has_permissions('edit', self) else 'anonymous'
    for asset in self.assets:
        h << asset.render(h, model)
    return h.root


@presentation.render_for(Gallery, 'crop')
def render_Gallery_crop(self, h, comp, model):
    return self.cropper.on_answer(self.action)


@presentation.render_for(Gallery, 'cover')
def render_cover(self, h, comp, model):
    cover = self.get_cover()
    if cover:
        h << h.p(component.Component(cover, model='cover'), class_='cover')
    return h.root


@presentation.render_for(Gallery, model='badge')
def render_gallery_badge(self, h, *args):
    """Gallery badge for the card"""
    num_assets = self.num_assets
    if num_assets:
        with h.span(class_='badge'):
            h << h.span(h.i(class_='icon-file-empty'), ' ', num_assets, class_='label')
    return h.root


@presentation.render_for(Gallery, "action")
def render_download(self, h, comp, *args):
    if security.has_permissions('edit', self):
        submit_id = h.generate_id("attach_submit")
        input_id = h.generate_id("attach_input")
        h << h.label((h.i(class_='icon-file-empty'),
                      _("Add file")), class_='btn', for_=input_id)
        with h.form:
            h << h.script(
                u'''
    function valueChanged(e) {
        if (YAHOO.kansha.app.checkFileSize(this, %(max_size)s)) {
            YAHOO.util.Dom.get(%(submit_id)s).click();
            YAHOO.kansha.app.showWaiter();
        } else {
            alert(%(error)s);
        }
    }

    YAHOO.util.Event.onDOMReady(function() {
        YAHOO.util.Event.on(%(input_id)s, 'change', valueChanged);
    });''' %
                {
                    'max_size': ajax.py2js(self.assets_manager.max_size),
                    'input_id': ajax.py2js(input_id),
                    'submit_id': ajax.py2js(submit_id),
                    'error': ajax.py2js(
                        _(u'Max file size exceeded')
                    ).decode('UTF-8')
                }
            )
            submit_action = ajax.Update(
                render=lambda r: r.div(comp.render(r, model=None), r.script('YAHOO.kansha.app.hideWaiter()')),
                component_to_update='gal' + self.comp_id,
            )

            h << h.input(id=input_id, class_='hidden', type="file", name="file", multiple="multiple", maxlength="100",).action(self.add_assets)
            h << h.input(class_='hidden', id=submit_id, type="submit").action(submit_action)
    return h.root


@presentation.render_for(Asset)
@presentation.render_for(Asset, model='thumb')
@presentation.render_for(Asset, model='medium')
@presentation.render_for(Asset, model='cover')
def render_asset(self, h, comp, model, *args):
    res = []
    metadata = self.assets_manager.get_metadata(self.filename)
    kw = {'randomize': True} if model == 'cover' else {}
    kw['class'] = model
    if self.is_cover:
        res.append(h.span(class_='is_cover'))

    meth = CONTENT_TYPES.get(metadata['content-type'], render_file)
    res.append(meth(self, h, comp, model, **kw))
    return res

@presentation.render_for(Asset, model='edit')
def render_Asset_thumb(self, h, comp, model, *args):
    with h.div(class_='asset'):
        action = h.a.action(lambda: comp.answer(('delete', self))).get('onclick')
        onclick = _(u'Are you sure you want to delete this file?')
        onclick = u'if (confirm("%s")) { %s }' % (onclick, action)
        with h.a(class_='delete', title=_(u'Delete'), href='#', onclick=onclick):
            h << h.i(class_='icon-cross')
        if self.is_image():
            with h.a(class_='cover', title=_(u'Configure cover')).action(lambda: comp.answer(('configure_cover', self))):
                if self.is_cover:
                    h << {'style': 'visibility: visible'}
                h << h.i(class_='icon-checkmark')
        with h.a(href=self.assets_manager.get_image_url(self.filename), target='_blank'):
            h << comp.render(h, 'thumb')
    return h.root

@presentation.render_for(Asset, model="anonymous")
def render_asset_anonymous(self, h, comp, model, *args):
    with h.div(class_='asset'):
        with h.a(href=self.assets_manager.get_image_url(self.filename), target='_blank'):
            h << comp.render(h, model="thumb")
    return h.root


@presentation.render_for(AssetCropper)
def render_gallery_cropper(self, h, comp, *args):
    h << h.p(_('Use the controls below to create the cover of your card.'))

    form_id = h.generate_id()
    img_id = h.generate_id()

    with h.form:
        for crop_name in 'crop_left', 'crop_top', 'crop_width', 'crop_height':
            h << h.input(type='hidden', id=form_id + '_' + crop_name).action(getattr(self, crop_name))
        h << h.p(render_image(self.asset, h, comp, 'medium', id=img_id))
        h << h.script(
            "YAHOO.util.Event.onContentReady(%s,"
            "function(){YAHOO.kansha.app.initCrop(%s, %s, %s, %s)})" % (
                ajax.py2js(img_id),
                ajax.py2js(img_id),
                ajax.py2js(form_id),
                ajax.py2js(self.crop_width()),
                ajax.py2js(self.crop_height())
            )
        )
        with h.div(class_='buttons'):
            h << h.button(_('Create cover'), class_='btn btn-primary').action(self.commit, comp)
            if self.asset.is_cover:
                h << ' '
                h << h.button(_('Remove cover'), class_='btn delete').action(self.remove_cover, comp)
            h << ' '
            h << h.button(_('Cancel'), class_='btn').action(self.cancel, comp)
    return h.root



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

import re
import unicodedata
from cStringIO import StringIO

import xlwt
from webob import exc
from nagare.i18n import _
from peak.rules import when

from kansha.cardextension import CardExtension


def get_extension_title(card_extension_class):
    '''Return a title (string) if the extension has to write something in Excel export, None otherwise'''


def write_extension_data(card_extension, worksheet, row, col, style):
    '''Write data to Excel sheet'''


def get_extension_title_for(cls):
    cond = 'card_extension_class.__name__ == "%s"' % cls.__name__
    return when(get_extension_title, cond)


def write_extension_data_for(cls):
    cond = 'isinstance(card_extension, %s)' % cls.__name__
    return when(write_extension_data, cond)


@when(get_extension_title, CardExtension)
def get_extension_title_CardExtension(card_extension):
    return None


@when(write_extension_data, CardExtension)
def write_extension_data_CardExtension(card_extension):
    pass


class ExcelExport(object):
    def __init__(self, board):
        self.board = board
        self.workbook = xlwt.Workbook()
        name = unicodedata.normalize('NFKD', self.board.get_title()).encode('ascii', 'ignore')
        self.filename = re.sub('\W+', '_', name.lower())
        self.sheet_name = re.sub('\W+', ' ', name)[:31]
        self.extensions = []
        self.extension_titles = []
        for name, cls in self.board.card_extensions.iteritems():
            title = get_extension_title(cls)
            if title is not None:
                self.extensions.append(name)
                self.extension_titles.append(title)

    def write(self):
        sty = ''
        header_sty = xlwt.easyxf(sty + 'font: bold on; align: wrap on, vert centre, horiz center;')
        sty = xlwt.easyxf(sty)
        ws = self.workbook.add_sheet(self.sheet_name)

        titles = [_(u'Column'), _(u'Title')] + self.extension_titles
        for col, title in enumerate(titles):
            ws.write(0, col, title, style=header_sty)

        row = 1
        for column in self.board.columns:
            column = column()
            colname = _('Archived cards') if column.is_archive else column.get_title()
            for card in column.cards:
                card = card()
                ws.write(row, 0, colname, sty)
                ws.write(row, 1, card.get_title(), sty)
                card_extensions = dict(card.extensions)
                for col, key in enumerate(self.extensions, 2):
                    ext = card_extensions[key]()
                    write_extension_data(ext, ws, row, col, sty)
                row += 1
        for col in xrange(len(titles)):
            ws.col(col).width = 0x3000
        ws.set_panes_frozen(True)
        ws.set_horz_split_pos(1)

    def download(self):
        buffer = StringIO()
        self.write()
        self.workbook.save(buffer)
        buffer.seek(0)
        e = exc.HTTPOk()
        e.content_type = 'application/vnd.ms-excel'
        e.content_disposition = u'attachment;filename=%s.xls' % self.filename
        e.body = buffer.getvalue()
        raise e



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.i18n import _
from nagare import presentation, security

from .comp import Votes


@presentation.render_for(Votes, model='action')
def render_Votes_edit(self, h, comp, *args):
    '''Add vote form'''
    if security.has_permissions('vote', self):
        if self.has_voted():
            msg = _('Unvote (%s)') % self.count_votes()
        else:
            msg = _('Vote (%s)') % self.count_votes()
        h << h.a(h.i(class_='icon-heart'), msg, class_='btn').action(self.toggle)
    return h.root


@presentation.render_for(Votes, model='badge')
def render_Votes_badge(self, h, *args):
    '''Badge for card in summary view'''
    nb_votes = self.count_votes()
    if nb_votes:
        with h.span(class_='badge'):
            h << h.span(h.i(class_='icon-heart'), ' ', nb_votes, class_='label')
    return h.root



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""board background

Revision ID: 36d972f11e3e
Revises: 1b6fbaffad32
Create Date: 2014-02-11 12:01:02.395563

"""

# revision identifiers, used by Alembic.
revision = '36d972f11e3e'
down_revision = '1b6fbaffad32'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('board', sa.Column('background_image', sa.Unicode))
    op.add_column('board', sa.Column('background_position', sa.Unicode, server_default='repeat'))
    op.add_column('board', sa.Column('title_color', sa.Unicode))
    op.execute('''update board set background_position='repeat' where background_position is null''')


def downgrade():
    op.drop_column('board', 'background_image')
    op.drop_column('board', 'background_position')
    op.drop_column('board', 'title_color')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import datetime

from nagare.i18n import _
from nagare import presentation, component
from nagare.namespaces.xhtml import absolute_url

from kansha import VERSION


class Header(object):
    def __init__(self, banner, theme):
        self.banner = banner
        self.theme = theme


@presentation.render_for(Header)
def render_Header(self, h, comp, *args):
    """Head renderer"""

    h.head << h.head.title(self.banner)
    h.head << h.head.meta(
        name='viewport', content='width=device-width, initial-scale=1.0')

    h.head.css_url('css/knacss.css')
    h.head.css_url('css/themes/fonts.css?v=2c')
    h.head.css_url('css/themes/kansha.css?v=2c')
    h.head.css_url('css/themes/login.css?v=2c')
    h.head.css_url('css/themes/%s/kansha.css?v=2c' % self.theme)
    h.head.css_url('css/themes/%s/login.css?v=2c' % self.theme)

    with h.div(class_='header'):
        with h.a(href=h.request.application_url):
            h << h.div(class_='logo')
        h << h.h1(self.banner)
    return h.root


class Login(object):
    def __init__(self, app_title, app_banner, favicon, theme, cfg, authentication_service, services_service):
        """Login components

        """

        self.app_title = app_title
        self.favicon = favicon
        logins = []
        for each in authentication_service:
            logins.append(services_service(authentication_service[each], app_title, app_banner, theme))
        self.logins = [component.Component(login) for login in logins]
        self.header = component.Component(
            Header(
                cfg['pub_cfg']['banner'],
                theme
            )
        )
        self.disclaimer = cfg['pub_cfg']['disclaimer']


@presentation.render_for(Login)
def render_Login(self, h, comp, *args):

    favicon_url = absolute_url(self.favicon, h.head.static_url)
    h.head << h.head.link(rel="icon", type="image/x-icon", href=favicon_url)
    h.head << h.head.link(rel="shortcut icon", type="image/x-icon", href=favicon_url)
    h.head << h.head.title(self.app_title)
    with h.body(class_='body-login slots%s' % len(self.logins)):
        h << self.header
        with h.div(class_='title'):
            title = _(u'Sign in')
            for login in self.logins:
                if login().alt_title:
                    title = login().alt_title
                    break
            h << h.h2(title)
            for login in self.logins:
                if login().error_message:
                    h << h.small(login().error_message, class_='error')
                    login().error_message = u''
        with h.div(class_='container'):
            for index, login in enumerate(self.logins, 1):
                h << login.on_answer(comp.answer)

        with h.div(class_='message'):
            h << h.parse_htmlstring(self.disclaimer) if self.disclaimer else u''
        with h.div(class_='credits'):
            h << h.a(self.app_title, href='http://www.kansha.org', target='_blank') << (u' v%s - \u00a9 ' % VERSION)
            h << h.a('Net-ng', href='http://www.net-ng.com', target='_blank') << (' %d' % datetime.date.today().year)

    return h.root



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from datetime import datetime, timedelta

from nagare import log
from nagare.database import session
from kansha.user import usermanager

APP_NAME = 'kansha'

UserConfirmationTimeout = timedelta(minutes=1)


def main():

    before_date = datetime.now() - UserConfirmationTimeout
    log.info('Clearing users that did not confirm their'
             ' email since %s...' % before_date)

    unconfirmed_users = list(
        usermanager.UserManager.get_unconfirmed_users(before_date))

    if not unconfirmed_users:
        log.info("No user found")
    else:
        for user in unconfirmed_users:
            for vote in user.votes:
                vote.delete()
            user.delete()
            log.info('- %s (%s) removed' % (user.fullname, user.username))

    session.commit()  # @UndefinedVariable


# call main
main()



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

import hashlib
from io import BytesIO
from datetime import datetime

import identicon

from nagare import i18n
from nagare.security import common as security_common

from .models import DataToken, DataUser


def int_hash(string):
    sha = hashlib.sha256()
    sha.update(string)
    return int(sha.hexdigest(), 16)


class User(security_common.User):
    """User component"""

    def __init__(self, username, *args, **kw):
        """Initialization

        In:
            - ``username`` -- the id of the user
        """
        super(User, self).__init__(username, *args)
        self.username = username
        self._data = kw.get('data')

    @property
    def data(self):
        """Return the user object from database
        """
        if not self._data:
            self._data = DataUser.get_by_username(self.username)
        return self._data

    def __getstate__(self):
        self._data = None
        return self.__dict__

    def __eq__(self, other):
        return self.username == other.username

    @property
    def email(self):
        return self.data.email or self.data.email_to_confirm

    @property
    def source(self):
        return self.data.source

    @property
    def is_local(self):
        """Test if it's a application user

        Return:
         - True if user is an application user
        """
        return self.data.source == 'application'

    def get_locale(self):
        locale = i18n.Locale(self.data.language)

        # At this point, the locale object only knows about builtin Nagare translation directories
        # We need to register Kansha's translation directories too (get them from the current locale)
        locale.translation_directories.update(i18n.get_locale().translation_directories)

        return locale

    def update_password(self, new_password):
        """Update the password of the connected user

        In:
         - ``new_password`` -- the new password
        """
        self.set_id(self.username, new_password)

    def set_last_board(self, board):
        """Update last board used

        In;
         - ``board`` -- last board used
        """
        self.data.last_board = board.data

    def get_last_board(self):
        """Return last used board

        In;
         - ``board`` -- last board used
        """
        return self.data.last_board

    def get_avatar(self):
        """Return picture

        Return:
         - URL or URI of the picture
        """
        return self.data.get_picture()

    def reset_avatar(self, assets_manager_service):
        avatar = identicon.render_identicon(int_hash(self.data.email or self.data.email_to_confirm),
                                            48)
        icon_file = BytesIO()
        avatar.save(icon_file, 'PNG')
        assets_manager_service.save(
            icon_file.getvalue(),
            self.data.username,
            {'filename': '%s.png' % self.data.username})
        self.data.picture = assets_manager_service.get_image_url(self.data.username, 'thumb')

    @property
    def fullname(self):
        return self.data.fullname

    @property
    def last_login(self):
        return self.data.last_login

    def update_last_login(self):
        self.data.last_login = datetime.utcnow()


class PendingUser(object):
    """ Class for pending user of a board

    Store token id, to get the invited user
    """

    def __init__(self, token_id):
        self.token_id = token_id

    @property
    def data(self):
        return DataToken.get(self.token_id)

    @property
    def username(self):
        return self.data.username

    @property
    def email(self):
        return self.data.username



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import datetime
import hashlib
import random
import string

from elixir import Unicode, Field, DateTime
from elixir import ManyToOne, OneToOne, OneToMany
from elixir import using_options

from sqlalchemy.dialects.mysql import VARCHAR

from kansha.models import Entity


class DataToken(Entity):
    using_options(tablename='token')
    token = Field(Unicode(255), unique=True, nullable=False, primary_key=True)
    username = Field(Unicode(255), nullable=True)
    date = Field(DateTime, nullable=False)
    action = Field(Unicode(255), nullable=True)
    board = ManyToOne('DataBoard')

    @classmethod
    def delete_by_username(cls, username, action):
        token = cls.get_by(username=username, action=action)
        if token:
            token.delete()

    @classmethod
    def new(cls, token, username, action):
        obj = cls(
            token=token,
            username=username,
            action=action,
            date=datetime.datetime.now()
        )
        return obj


class DataUser(Entity):

    """Label mapper
    """
    using_options(tablename='user')
    # VARCHAR(binary=True) here is a hack to make MySQL case sensitive
    # like the other DBMS.
    # No consequences on regular databases.
    username = Field(
        VARCHAR(255, binary=True), unique=True,
        primary_key=True, nullable=False)
    source = Field(Unicode(255), nullable=False, primary_key=True)
    fullname = Field(Unicode(255), nullable=False)
    email = Field(Unicode(255), nullable=True, unique=True, index=True)
    picture = Field(Unicode(255), nullable=True)
    language = Field(Unicode(255), default=u"en", nullable=True)
    email_to_confirm = Field(Unicode(255))
    _salt = Field(Unicode(255), colname='salt', nullable=False)
    _password = Field(Unicode(255), colname='password', nullable=True)
    registration_date = Field(DateTime, nullable=False)
    last_login = Field(DateTime, nullable=True)
    last_board = OneToOne('DataBoard', inverse='last_users')
    # history = OneToMany('DataHistory')

    def __init__(self, username, password, fullname, email,
                 source=u'application', picture=None, **kw):
        """Create a new user with an unconfirmed email"""
        super(DataUser, self).__init__(username=username,
                                       fullname=fullname,
                                       email=None,
                                       email_to_confirm=email,
                                       source=source,
                                       picture=picture,
                                       registration_date=datetime.datetime.utcnow(), **kw)
        # Create password if source is local
        if source == "application":
            self.change_password(password)
        else:
            # External authentication
            self.change_password('passwd')
            self.email_to_confirm = None

    @property
    def id(self):
        return self.username

    def update(self, fullname, email, picture=None):
        self.fullname = fullname
        if email:
            self.email = email
        self.picture = picture

    def check_password(self, clear_password):
        """Check the user password. Return True if the password is valid for this user"""
        encrypted_password = self._encrypt_password(self._salt, clear_password)
        return encrypted_password == self._password

    def change_password(self, clear_password):
        """Change the user password"""
        self._salt = self._create_random_salt()
        self._password = self._encrypt_password(self._salt, clear_password)

    def set_email_to_confirm(self, email_to_confirm):
        if email_to_confirm:
            self.email_to_confirm = email_to_confirm

    def is_validated(self):
        return self.email_to_confirm is None

    def confirm_email(self):
        """Called when a user confirms his email address"""
        # already confirmed
        if self.email_to_confirm is None:
            return

        self.email = self.email_to_confirm
        self.email_to_confirm = None

    def get_picture(self):
        return self.picture

    @classmethod
    def get_confirmed_users(cls):
        return cls.query.filter(cls.email is not None)

    @staticmethod
    def _create_random_salt(length=32):
        allowed_chars = string.ascii_letters + string.digits
        return u''.join(random.choice(allowed_chars) for _ in range(length))

    @staticmethod
    def _encrypt_password(salt, password):
        secret = "NzlSszmvDNY2e2lVMwiKJwgWjNGFCP1a"
        secret_salt = hashlib.sha512(secret + salt).hexdigest()
        utf8_password = password.encode('utf-8')
        return unicode(hashlib.sha512(secret_salt + utf8_password).hexdigest())

    @classmethod
    def get_unconfirmed_users(cls, before_date=None):
        q = cls.query.filter(cls.email is None)
        if before_date:
            q = q.filter(cls.registration_date < before_date)
        return q

    @classmethod
    def get_by_username(cls, username):
        return cls.get_by(username=username)

    @classmethod
    def get_by_email(cls, email):
        return cls.get_by(email=email)

    @classmethod
    def search(cls, value):
        return cls.query.filter(cls.fullname.ilike('%' + value + '%') |
                                cls.email.ilike('%' + value + '%') |
                                cls.email_to_confirm.ilike('%' + value + '%'))



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.i18n import _
from peak.rules import when
from nagare import component, editor, validator, security

from kansha.board import excel_export
from kansha.cardextension import CardExtension
from kansha.services.actionlog.messages import render_event

from .models import DataCardWeight


@when(render_event, "action=='card_weight'")
def render_event_card_weight(action, data):
    return _(u'User %(author)s has weighted card "%(card)s" from (%(from)s) to (%(to)s)') % data


class CardWeightEditor(CardExtension):

    """ Card weight Form
    """

    LOAD_PRIORITY = 80

    # WEIGHTING TYPES
    WEIGHTING_OFF = 0
    WEIGHTING_FREE = 1
    WEIGHTING_LIST = 2

    def __init__(self, card, action_log, configurator):
        """
        In:
         - ``target`` -- Card instance
        """
        CardExtension.__init__(self, card, action_log, configurator)
        self.card = card
        self.weight = editor.Property(str(self.data.weight or u''))
        self.weight.validate(self.validate_weight)
        self.action_button = component.Component(self, 'action_button')

    def update(self, other):
        self.data.update(other.data)
        self.weight(str(self.data.weight) or u'')

    @property
    def data(self):
        data = DataCardWeight.get_by_card(self.card.data)
        if data is None:
            data = DataCardWeight.new(card=self.card.data)
        return data

    def validate_weight(self, value):
        """
        Integer or empty
        """
        if value:
            value = validator.IntValidator(value).greater_or_equal_than(0).to_int()
        else:
            value = 0
        return value

    @property
    def weighting_switch(self):
        return self.configurator.weighting_cards if self.configurator else self.WEIGHTING_OFF

    @property
    def allowed_weights(self):
        return self.configurator.weights if self.configurator else ''

    def commit(self):
        success = False
        if self.weight.error is None:
            values = {'from': self.data.weight, 'to': self.weight.value, 'card': self.card.get_title()}
            self.data.weight = self.weight.value
            self.action_log.add_history(security.get_user(), u'card_weight', values)
            success = True
        return success

    def delete(self):
        self.data.delete()


@excel_export.get_extension_title_for(CardWeightEditor)
def get_extension_title_CardWeightEditor(card_extension):
    return _(u'Weight')


@excel_export.write_extension_data_for(CardWeightEditor)
def write_extension_data_CardWeightEditor(self, sheet, row, col, style):
    sheet.write(row, col, self.weight(), style)



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

from nagare.i18n import _
from nagare import ajax, component, presentation, security, var

from kansha import events

from .comp import Card, NewCard


@presentation.render_for(Card, 'no_dnd')
def render_card_no_dnd(self, h, comp, *args):
    """No DnD wrapping of the card"""
    return comp.render(h.AsyncRenderer())


@presentation.render_for(Card, 'new')
def render_card_new(self, h, comp, *args):
    h << comp.becomes(model=None)
    h << h.script(
        "card = YAHOO.util.Dom.get(%s);"
        "list = YAHOO.util.Dom.getAncestorByClassName(card, 'list-body');"
        "list.scrollTop = card.offsetTop - list.offsetTop;" %
        ajax.py2js(self.id)
    )
    return h.root


@presentation.render_for(Card)
def render(self, h, comp, *args):
    """Render the card"""

    extensions = self.extensions

    card_id = h.generate_id()

    onclick = h.a.action(self.emit_event, comp, events.CardClicked, comp).get('onclick').replace('return', "")
    with h.div(id=self.id, class_='card ' + self.card_filter(self)):
        with h.div(id=card_id, onclick=onclick):
            with h.div(class_='headers'):
                h << [extension.render(h, 'header') for _name, extension in extensions]
            with h.div(class_='covers'):
                with h.div(class_='title'):
                    h << self.title.render(h, 'readonly')
                h << [extension.render(h, 'cover') for _name, extension in extensions]
            with h.div(class_='badges'):
                h << [extension.render(h, 'badge') for _name, extension in extensions]
    if self.card_filter(self):
        h << component.Component(self.card_filter)

    h << h.script(
        "YAHOO.kansha.reload_cards[%s]=function() {%s}""" % (
            ajax.py2js(self.id),
            h.a.action(ajax.Update()).get('onclick')
        )
    )

    return h.root


@presentation.render_for(Card, model='readonly')
def render(self, h, comp, *args):
    """Render the card read-only"""
    with h.div(id=self.id, class_='card'):
        with h.div:
            with h.div(class_='title'):
                h << self.title.render(h, 'readonly')
            # FIXME: unify with main card view.
    return h.root


@presentation.render_for(Card, model='edit')
def render_card_edit(self, h, comp, *args):
    """Render card in edition mode"""
    # Test for delete card
    if self.data is None:
        return h.root

    parent_title = self.emit_event(comp, events.ParentTitleNeeded) or ''

    with h.div(class_='card-edit-form'):
        with h.div(class_='header'):
            with h.div(class_='title'):
                h << self.title.render(h.AsyncRenderer(), 0 if security.has_permissions('edit', self) else 'readonly')
                h << h.span('(%s)' % parent_title, class_='reminder')
        with h.div:
            with h.div(class_='card-actions'):
                h << comp.render(h, 'delete-action')
                h << [extension.render(h.AsyncRenderer(), 'action') for __, extension in self.extensions]
            with h.div(class_='card-edition'):
                for name, extension in self.extensions:
                    h << h.div(extension.render(h.AsyncRenderer()), class_=name)
        h << h.div(class_='clear')
    return h.root


@presentation.render_for(Card, 'delete-action')
def render_card_delete(self, h, comp, model):
    if security.has_permissions('edit', self) and not self.archived:
        with h.form:
            h << h.SyncRenderer().button(
                h.i(class_='icon-bin'),
                _('Delete'),
                onclick='return confirm(%s)' % ajax.py2js(
                    _(u'This card will be archived. Are you sure?')
                ).decode('UTF-8'),
                class_='btn delete').action(self.emit_event, comp, events.CardArchived)
    return h.root


@presentation.render_for(Card, 'calendar')
def render_in_calendar(self, h, comp, *args):
    # TODO should be in due_date extension
    due_date = dict(self.extensions)['due_date']().due_date
    if due_date:
        due_date = ajax.py2js(due_date, h)
        parent_title = self.emit_event(comp, events.ParentTitleNeeded) or ''
        card = u'{title:%s, editable:true, allDay: true, start: %s, _id: %s}' % (
            ajax.py2js(u'{} ({})'.format(self.data.title, parent_title), h).decode('utf-8'),
            due_date, ajax.py2js(self.id, h))
        clicked_cb = h.a.action(
            lambda: self.emit_event(comp, events.CardClicked, comp)
        ).get('onclick')

        dropped_cb = h.a.action(
            ajax.Update(
                action=self.card_dropped,
                render=lambda render: '',
                with_request=True
            )
        ).get('onclick')[:-2]

        h << h.script(u"""YAHOO.kansha.app.add_event($('#calendar'), %(card)s,
                         function() { %(clicked_cb)s},
                         function(start) { %(dropped_cb)s&start="+start);} )""" % {
            'card': card,
            'clicked_cb': clicked_cb,
            'dropped_cb': dropped_cb
        })

    return h.root


@presentation.render_for(Card, 'dnd')
def render_card_dnd(self, h, comp, *args):
    """DnD wrapping of the card"""
    if self.data is not None:
        id_ = h.generate_id('dnd')
        with h.div(id=id_, class_='card-dnd-wrapper'):
            h << comp.render(h.AsyncRenderer())
            h << h.script('YAHOO.kansha.dnd.initCard(%s)' % ajax.py2js(id_))
    return h.root


#####


@presentation.render_for(NewCard)
def render_new_card(self, h, comp, *args):
    """Render card creator minified"""
    h << h.a(h.strong('+'), h.span(_('Add a card')),
             class_='link-small').action(comp.becomes, model='add')
    if self.needs_refresh:
        h << h.script('increase_version();')
        self.toggle_refresh()
    return h.root


def toggle_answer(card_editor, comp, text):
    card_editor.toggle_refresh()
    comp.becomes(model=None)
    if text:
        comp.answer(text().strip())


@presentation.render_for(NewCard, 'add')
def render_new_card_add(self, h, comp, *args):
    """Render card creator form"""
    text = var.Var()
    id_ = h.generate_id('newCard')

    with h.form(class_='card-add-form'):
        h << h.input(type='text', id=id_).action(text)
        h << h.button(_('Add'), class_='btn btn-primary').action(toggle_answer, self, comp, text)
        h << ' '
        h << h.button(_('Cancel'), class_='btn').action(toggle_answer, self, comp, None)

    h << h.script("""document.getElementById(%s).focus(); """ % ajax.py2js(id_))

    return h.root



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from datetime import date, timedelta

from kansha.cardextension.tests import CardExtensionTestCase

from .comp import DueDate

class CardDueDateTest(CardExtensionTestCase):

    extension_name = 'due_date'
    extension_class = DueDate

    def test_due_date(self):
        today = date.today()
        self.assertIsNone(self.extension.due_date)
        self.assertIsNone(self.extension.get_value())
        self.extension.set_value(today)
        self.assertEqual(self.extension.get_value(), today)

    def test_days_count(self):
        today = date.today()
        self.extension.set_value(today)
        self.assertEqual(self.extension.get_days_count(), 0)
        self.extension.set_value(today + timedelta(days=5))
        self.assertEqual(self.extension.get_days_count(), -5)
        self.extension.set_value(today - timedelta(days=5))
        self.assertEqual(self.extension.get_days_count(), 5)

    def test_class(self):
        today = date.today()
        self.extension.set_value(today)
        self.assertEqual(self.extension.get_class(), 'today')
        self.extension.set_value(today + timedelta(days=1))
        self.assertEqual(self.extension.get_class(), 'tomorrow')
        self.extension.set_value(today + timedelta(days=10))
        self.assertEqual(self.extension.get_class(), 'future')
        self.extension.set_value(today - timedelta(days=1))
        self.assertEqual(self.extension.get_class(), 'yesterday')
        self.extension.set_value(today - timedelta(days=10))
        self.assertEqual(self.extension.get_class(), 'past')



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.i18n import _
from peak.rules import when
from nagare import component, security, log

from kansha import exceptions
from kansha.toolbox import overlay
from kansha.user import usermanager
from kansha.cardextension import CardExtension
from kansha.services.actionlog.messages import render_event

from .models import DataMembership, DataCardMembership


@when(render_event, "action=='card_add_member'")
def render_event_card_add_member(action, data):
    return _(u'User %(user)s has been assigned to card "%(card)s"') % data


@when(render_event, "action=='card_remove_member'")
def render_event_card_remove_member(action, data):
    return _(u'User %(user)s has been unassigned from card "%(card)s"') % data


class Membership(object):

    def __init__(self, data):
        self.id = data.id
        self._data = data

    @property
    def data(self):
        """Return the board object from the database
        PRIVATE
        """
        if self._data is None:
            self._data = DataMembership.get(self.id)
        return self._data

    def __getstate__(self):
        self._data = None
        return self.__dict__

    @property
    def notify(self):
        return self.data.notify

    @notify.setter
    def notify(self, level):
        self.data.notify = level

    @classmethod
    def search(cls, board, user):
        data = DataMembership.search(board.data, user.data)
        return cls(data) if data else None

    @classmethod
    def subscribers(cls):
        return (cls(data) for data in DataMembership.subscribers())


class CardMembers(CardExtension):

    LOAD_PRIORITY = 90

    MAX_SHOWN_MEMBERS = 3

    def __init__(self, card, action_log, configurator):
        """
        Card is a card business object.
        """

        super(CardMembers, self).__init__(card, action_log, configurator)

        # members part of the card
        self.overlay_add_members = component.Component(
            overlay.Overlay(lambda r: (r.i(class_='ico-btn icon-user-plus')),
                            lambda r: component.Component(self).render(r, model='add_member_overlay'), dynamic=True, cls='card-overlay'))
        self.new_member = component.Component(usermanager.NewMember(self.autocomplete_method), model='add_members')
        self.members = [component.Component(usermanager.UserManager.get_app_user(data=membership.user))
                        for membership in DataMembership.get_for_card(self.card.data)]

        self.see_all_members = component.Component(
            overlay.Overlay(lambda r: component.Component(self).render(r, model='more_users'),
                            lambda r: component.Component(self).on_answer(self.remove_member).render(r, model='members_list_overlay'),
                            dynamic=False, cls='card-overlay'))
        self._favorites = []

    def autocomplete_method(self, value):
        """ """
        available_user_ids = self.get_available_user_ids()
        return [u for u in usermanager.UserManager.search(value) if u.id in available_user_ids]

    def get_available_user_ids(self):
        """Return ids of users who are authorized to be added on this card

        Return:
            - a set of ids
        """
        return self.get_all_available_user_ids() - set(user().id for user in self.members)

    def get_all_available_user_ids(self):
        return self.configurator.get_available_user_ids() if self.configurator else []

    @property
    def favorites(self):
        """Return favorites users for a given card

        Return:
            - list of favorites (User instances) wrapped in component
        """

        # to be optimized later if still exists
        member_usernames = set(member().username for member in self.members)
        # Take the 5 most popular that are not already affected to this card
        favorites = [userdata for userdata in DataMembership.favorites_for(self.card.data)
                     if userdata.username not in member_usernames]

        # store component for callback lookup
        self._favorites = [component.Component(usermanager.UserManager.get_app_user(data=userdata), "friend")
                           for userdata in favorites[:5]]
        return self._favorites

    def add_members(self, emails):
        """Add new members from emails

        In:
            - ``emails`` -- list of strings
        """

        memberships = DataMembership.add_card_members_from_emails(self.card.data, emails)

        for member in (usermanager.UserManager.get_app_user(data=ms.user) for ms in memberships):
            values = {'user_id': member.username, 'user': member.fullname, 'card': self.card.get_title()}
            self.action_log.add_history(security.get_user(), u'card_add_member', values)
            self.members.append(component.Component(member))

    def remove_member(self, username):
        """Remove member username from card member"""
        DataMembership.remove_card_member(self.card.data, username)
        for member in self.members:
            if member().username == username:
                self.members.remove(member)
                values = {'user_id': member().username, 'user': member().data.fullname,
                          'card': self.card.get_title()}
                self.action_log.add_history(security.get_user(), u'card_remove_member', values)
                break

    def has_permission_on_card(self, user, perm):
        granted = True
        if perm == 'edit':
            granted = user and (user.id in self.get_all_available_user_ids())
        return granted

    def delete(self):
        DataCardMembership.purge(self.card.data)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import presentation, security, log, component
from nagare.i18n import _

from kansha.exceptions import NotFound
from kansha.authentication.database import forms
from kansha.user.usermanager import UserManager

from .comp import MainTask


def logout():
    if security.get_user() is not None:
        security.get_manager().logout()


@presentation.init_for(MainTask, "len(url) == 2")
def init_app__board(self, url, comp, http_method, request):
    kansha = self.app
    kansha.select_board_by_uri(url[1])


@presentation.init_for(MainTask, "url[0] == 'logout'")
def init_app__logout(self, url, comp, http_method, request):
    logout()


@presentation.init_for(MainTask)
def init_app(self, url, comp, http_method, request):
    """
    Forward everything to the MainTask to maintain compatibility and catch
    404 errors to use custom error page
    """
    log.error('url "%s" not found' % request.url)
    raise NotFound()


@presentation.init_for(MainTask, "len(url) == 3 and url[0] == 'register'")
def init_app__register(self, url, comp, http_method, request):
    register = self._services(
        forms.RegistrationTask,
        self.app_title,
        self.app_banner,
        self.theme
    )
    register.state = (url[1], url[2])
    comp.becomes(register).on_answer(lambda v: logout())


@presentation.init_for(MainTask, "len(url) == 3 and url[0] == 'new_mail'")
def init_app__new_mail(self, url, comp, http_method, request):
    username, token = (url[1], url[2])
    get_user = lambda: UserManager.get_by_username(username)
    confirmation = self._services(
        forms.EmailConfirmation,
        self.app_title,
        self.app_banner,
        self.theme,
        get_user
    )
    if confirmation.confirm_email_address(token):
        log.debug(_('Change email success for user %s') % get_user().username)
        comp.becomes(self._services(
            forms.ChangeEmailConfirmation,
            self.app_title,
            self.app_banner,
            self.theme,
            request.application_url
        ),
            model='success'
        )
        confirmation.reset_token(token)
    else:
        log.debug(_('Change email failure for user %s') % get_user().username)
        comp.becomes(
            self._services(
                forms.ChangeEmailConfirmation,
                self.app_title,
                self.app_banner,
                self.theme,
                request.application_url
            ),
            model='failure'
        )


@presentation.init_for(MainTask, "len(url) == 3 and url[0] == 'reset'")
def init_app__reset(self, url, comp, http_method, request):
    reset = self._services(
        forms.PasswordResetTask,
        self.app_title,
        self.app_banner,
        self.theme,
    )
    reset.state = (url[1], url[2], request.application_url)
    comp.becomes(reset).on_answer(lambda v: logout())


@presentation.init_for(MainTask, "len(url) >= 3 and url[0] == 'services'")
def init_app__assets(self, url, comp, http_method, request):
    if url[1] in self._services:
        component.Component(self._services[url[1]]).init(url[2:], http_method, request)
    else:
        raise NotFound()
